#encoding...
""" Module documentation """
#importing the relations
from __rels__ import _rel_Database_Schema_2 
from __rels__ import _rel_Schema_Table_1 
from __rels__ import _rel_Table_Column_1 
from __rels__ import _rel_Table_ForeignKey_2 
from __rels__ import _rel_ForeignKey_Table_1 
from __rels__ import _rel_ForeignKey_Table_2 


class Database:
  """ 
 Container(s)  : 
  >>> database = Database()  
  >>> database.name = "..."
  >>> database.url = "..."
  >>> database.ownedSchemas.append(Schema())
  >>> schema  = database.ownedSchemas[0]

  """

  def __init__(self):
    self._parent_ = None
    self._meta_ = self.__class__
    self.name = "..."
    self.url = "..."

    self.ownedSchemas = _rel_Database_Schema_2(parent=self)
    self.ownedSchemas._parent_ = self


class Schema:
  """ 
 Container(s)  : Database
  >>> schema = Schema()  
  >>> schema.name = "..."
  >>> schema.ownedTables.append(Table())
  >>> table  = schema.ownedTables[0]

  """

  def __init__(self):
    self._parent_ = None
    self._meta_ = self.__class__
    self.name = "..."

    self.ownedTables = _rel_Schema_Table_1(parent=self)
    self.ownedTables._parent_ = self


class Table:
  """ 
 Container(s)  : Schema
  >>> table = Table()  
  >>> table.name = "..."
  >>> table.ownedColumns.append(Column())
  >>> column  = table.ownedColumns[0]
  >>> table.ownedForeignKeys.append(ForeignKey())
  >>> foreignkey  = table.ownedForeignKeys[0]

  """

  def __init__(self):
    self._parent_ = None
    self._meta_ = self.__class__
    self.name = "..."

    self.ownedColumns = _rel_Table_Column_1(parent=self)
    self.ownedColumns._parent_ = self
    self.ownedForeignKeys = _rel_Table_ForeignKey_2(parent=self)
    self.ownedForeignKeys._parent_ = self


class ForeignKey:
  """ 
 Container(s)  : Table
  >>> foreignkey = ForeignKey()  
  >>> foreignkey.name = "..."
  >>> foreignkey.foreignTable.append(Table())
  >>> table  = foreignkey.foreignTable[0]
  >>> foreignkey.sourceTable.append(Table())
  >>> table  = foreignkey.sourceTable[0]

  """

  def __init__(self):
    self._parent_ = None
    self._meta_ = self.__class__
    self.name = "..."

    self.foreignTable = _rel_ForeignKey_Table_1(parent=self)
    self.foreignTable._parent_ = self
    self.sourceTable = _rel_ForeignKey_Table_2(parent=self)
    self.sourceTable._parent_ = self


class Column:
  """ 
 Container(s)  : Table
  >>> column = Column()  
  >>> column.name = "..."
  >>> column.type = None
  >>> column.isPrimaryKey = False
  >>> column.isUnique = False


  """

  def __init__(self):
    self._parent_ = None
    self._meta_ = self.__class__
    self.name = "..."
    self.type = None
    self.isPrimaryKey = False
    self.isUnique = False



class Type:
  """ 

  """


import unittest

class Testrelational(unittest.TestCase):

  def testDatabaseAttributes(self):
	  database = Database()  
	  database.name = "..."
	  self.assertEquals(database.name,"...")
	  database.url = "..."
	  self.assertEquals(database.url,"...")

  def testSchemaAttributes(self):
	  schema = Schema()  
	  schema.name = "..."
	  self.assertEquals(schema.name,"...")

  def testTableAttributes(self):
	  table = Table()  
	  table.name = "..."
	  self.assertEquals(table.name,"...")

  def testForeignKeyAttributes(self):
	  foreignkey = ForeignKey()  
	  foreignkey.name = "..."
	  self.assertEquals(foreignkey.name,"...")

  def testColumnAttributes(self):
	  column = Column()  
	  column.name = "..."
	  self.assertEquals(column.name,"...")
	  column.type = None
	  self.assertEquals(column.type,None)
	  column.isPrimaryKey = False
	  self.assertEquals(column.isPrimaryKey,False)
	  column.isUnique = False
	  self.assertEquals(column.isUnique,False)


  def testDatabaseRelations_rel_Database_Schema_2(self):
      database = Database()  
      #testing _rel_Database_Schema_2      
      #checking unicity support
      schema1 = Schema()
      database.ownedSchemas.append(schema1)
      self.assertRaises(Exception,database.ownedSchemas.append,schema1)
	  #checking upper_bound limit

  def testSchemaRelations_rel_Schema_Table_1(self):
      schema = Schema()  
      #testing _rel_Schema_Table_1      
      #checking unicity support
      table1 = Table()
      schema.ownedTables.append(table1)
      self.assertRaises(Exception,schema.ownedTables.append,table1)
	  #checking upper_bound limit

  def testTableRelations_rel_Table_Column_1(self):
      table = Table()  
      #testing _rel_Table_Column_1      
      #checking unicity support
      column1 = Column()
      table.ownedColumns.append(column1)
      self.assertRaises(Exception,table.ownedColumns.append,column1)
	  #checking upper_bound limit
      table_limit = Table()  
      for i in range(0,4):
        table.ownedColumns.append(Column())
      self.assertRaises(Exception,table.ownedColumns.append,Column())

  def testTableRelations_rel_Table_ForeignKey_2(self):
      table = Table()  
      #testing _rel_Table_ForeignKey_2      
      #checking unicity support
      foreignkey1 = ForeignKey()
      table.ownedForeignKeys.append(foreignkey1)
      self.assertRaises(Exception,table.ownedForeignKeys.append,foreignkey1)
	  #checking upper_bound limit

  def testForeignKeyRelations_rel_ForeignKey_Table_1(self):
      foreignkey = ForeignKey()  
      #testing _rel_ForeignKey_Table_1      
      #checking unicity support
      table1 = Table()
      foreignkey.foreignTable.append(table1)
      self.assertRaises(Exception,foreignkey.foreignTable.append,table1)
	  #checking upper_bound limit
      foreignkey_limit = ForeignKey()  
      for i in range(0,0):
        foreignkey.foreignTable.append(Table())
      self.assertRaises(Exception,foreignkey.foreignTable.append,Table())

  def testForeignKeyRelations_rel_ForeignKey_Table_2(self):
      foreignkey = ForeignKey()  
      #testing _rel_ForeignKey_Table_2      
      #checking unicity support
      table1 = Table()
      foreignkey.sourceTable.append(table1)
      self.assertRaises(Exception,foreignkey.sourceTable.append,table1)
	  #checking upper_bound limit
      foreignkey_limit = ForeignKey()  
      for i in range(0,0):
        foreignkey.sourceTable.append(Table())
      self.assertRaises(Exception,foreignkey.sourceTable.append,Table())



if __name__ == '__main__':
    unittest.main()
"""

Module relational

Sample usage :

  >>> from relational import Database
  >>> database = Database()  
  >>> database.name = "..."
  >>> database.url = "..."
  >>> database.ownedSchemas.append(Schema())
  >>> schema  = database.ownedSchemas[0]


"""

from relational import *