<%--
  Author     Pascal Lindelauf
  E-Mail     plindelauf@yellowtail.nl
  Copyright  2007 Yellowtail
  All rights reserved.   This program and the accompanying materials
  are made available under the terms of the GNU Public License v2.0 or any later version
  which accompanies this distribution, and is available at
  http://www.gnu.org/licenses/gpl.txt
    You can apply any license to the files generated with this template
  and Acceleo.
--%>
<%--
  Target project: DomainModel
  This template generates a NHibernate mapping file for each UML class 
  stereotyped as "entityObject", that is located in a package that 
  is stereotyped as "domainModule" and that is not a child of any
  other class. 
  Known issues:
  * inheritance is not implemented yet
--%>
<%
metamodel http://www.obeo.fr/acceleo/uml14

import com.yellowtail.uml14.gen.csharp.services.StringService
import com.yellowtail.uml14.gen.csharp.services.NamespaceService
import com.yellowtail.uml14.gen.csharp.services.ClassService
import com.yellowtail.uml14.gen.csharp.services.AttributeService
import com.yellowtail.uml14.gen.csharp.services.AssociationService
import com.yellowtail.uml14.gen.csharp.services.Uml14Service
%>
<%-- TO DO: MEANINGFUL SELECTION OF SET/MAP/LIST/ARRAY, MULTIPLICITY IN THE ATTRIBUTES, INHERITANCE, INTRA & INTERMODULE --%>
<%script type="core.Class" name="filename"%>
<%-- Generate an NHibernate mapping file only for entity objects that are not a child in an inheritance hierarchy --%>
<%if (hasStereotype("entityObject")&&generalization.nSize()==0){%>
<%namespace.getRootContainer().name%>/DomainModel/<%namespace.name%>/DomainPersistence/<%name%>.hbm.xml
<%}%>

<%script type="core.Class" name="domain-class-hbm" file="<%filename%>"%>
<?xml version="1.0" encoding="utf-8" ?>
<hibernate-mapping xmlns="urn:nhibernate-mapping-2.0"
  namespace="<%getRootContainer().name%>.DomainModel.<%XLevelAndDownClassPath(2,".")%>"
  assembly="<%namespace.getRootContainer().name%>DomainModel">
  
  <class name="<%name%>" table="<%tableName%>">
  <%generateMappingBody%>
  </class>
</hibernate-mapping>

<%script type="core.Class" name="generateMappingBody"%>
<%-- START WITH CREATING THE IDENTIFIER, IF IT'S NOT NATIVELY PRESENT --%>
<%if (!thisContainsAnIdentifier()){%>
<%if (isChild()){%>
  <key column="id">
<%}else{%>
  <id name="Id" column="id" type="int" unsaved-value="0">
    <generator class="identity" />
  </id>
<%}%>
<%}else{%>
<%-- NOW GENERATE THE MAPPING OF THE NATIVELY PRESENT IDENTIFIER --%>
<%for (attribute){%>
<%if (isIdentifier){%>
<%if (owner().isChild()){%>
  <key column="<%columnName%>">
<%}else{%>
  <id name="<%propertyName%>" column="<%columnName%>" type="int" unsaved-value="0">
    <generator class="identity" />
  </id>
<%}%><%}%><%}%>
<%}%>

<%-- NOW GENERATE THE MAPPINGS OF THE NATIVELY PRESENT ATTRIBUTES OF A SINGULAR TYPE --%>
<%for (attribute){%>
<%if (!isIdentifier && !type.hasStereotype("valueObject")){%>
  <property name="<%propertyName%>" column="<%columnName%>"/>
<%}%>
<%}%>

<%-- NOW GENERATE THE MAPPINGS OF THE NATIVELY PRESENT ATTRIBUTES OF A VALUE OBJECT TYPE --%>
<%for (attribute){%>
<%if (!isIdentifier && type.hasStereotype("valueObject")){%>
  <%generateXToOneValueObject(name)%>
<%}%>
<%}%>

<%-- GENERATE THE MAPPINGS OF THE ASSOCIATIONS --%>
<%for (association){%>
  <%generateXToOne%><%generateXToMany%><%generateOneToOne%><%generateOneToMany%><%generateManyToOne%><%generateManyToMany%>
<%}%>

<%-- INHERITANCE IS NEXT --%>
<%-- FOR NOW USE THE "ONE TABLE PER HIERARCHY" STRATEGY ONLY --%>
<%for (children()){%>
  <%generateSubclass%>
<%}%>

<%script type="core.Class" name="generateSubclass"%>
  <joined-subclass name="<%name%>" table="<%tableName%>">
  <%generateMappingBody%>
  </joined-subclass>

<%script type="core.Attribute" name="generateXToOneValueObject"%>
<%-- ASSUMPTIONS: THE VALUE OBJECT HAS NO IDENTIFIER & IT HAS NO ASSOCIAITONS --%>
<%if (attributeUpperBound()==1){%>
	<component 
	  name="<%propertyName%>"                                
	  class="<%type.qualifiedName%>, 
	      	<%type.namespace.getRootContainer().name%>DomainModel"                              
	  <%-- insert="true|false" --%>                         <%-- Options: true|false --%> 
	  <%-- upate="true|false" --%>                          <%-- Options: true|false --%> 
	  <%-- access="field|property|nosetter|ClassName" --%>  <%-- Options: field|property|nosetter|ClassName --%>
	  <%-- optimistic-lock="true|false" --%>                <%-- Options: true|false --%>
	>
	  <%for (type.attribute){%>
	  <%if (!isIdentifier && !type.hasStereotype("valueObject")){%>
	  <property name="<%propertyName%>" column="<%args(0).firstLower()%><%columnName.firstUpper()%>"/>
	  <%}%>
	  <%}%>

	  <%for (type.attribute){%>
	  <%if (!isIdentifier && type.hasStereotype("valueObject")){%>
	  <%generateXToOneValueObject%>
	  <%}%>
	  <%}%>
	</component>
<%}%>

<%script type="core.AssociationEnd" name="generateXToOne"%>
<%if (!isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1){%>
	<many-to-one
	  name="<%associationPropertyName%>"
	  column="<%targetAssociationEnd.participant.foreignKeyName(targetAssociationEnd.name)%>"
	  class="<%targetAssociationEnd.participant.qualifiedName%>, 
	  	<%targetAssociationEnd.participant.namespace.getRootContainer().name%>DomainModel"
	  not-null="false" 
	  cascade="<%cascadeMode%>"			<%-- Options: all|none|save-update|delete --%> 
      <%-- fetch="join|select" --%>                   <%-- Options: join|select --%> 
      <%-- update="true|false" --%>                   <%-- Options: true|false --%> 
      <%-- insert="true|false" --%>                   <%-- Options: true|false --%> 
      <%-- property-ref="PropertyNameFromAssociatedClass" --%>
      access="property"  								<%-- Options: field|property|nosetter|ClassName --%>     
      <%-- unique="true|false" --%>                   <%-- Options: true|false --%> 
	/>
<%}%>

<%script type="core.AssociationEnd" name="generateXToMany"%>
<%if (!isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1){%>
	<set name="<%associationPropertyName%>">
	    <key column="<%targetAssociationEnd.participant.identifierColumnName%>"/>
	    <one-to-many
	      class="<%targetAssociationEnd.participant.qualifiedName%>, 
		      	 <%targetAssociationEnd.participant.namespace.getRootContainer().name%>DomainModel"/>
	</set>
<%}%>

<%script type="core.AssociationEnd" name="generateOneToOne"%>
<%if ((isNavigable && upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
	<one-to-one
        name="<%associationPropertyName%>"
		class="<%targetAssociationEnd.participant.qualifiedName%>, 
	      	<%targetAssociationEnd.participant.namespace.getRootContainer().name%>DomainModel"
        cascade="cascadeMode"				            <%-- Options: all|none|save-update|delete --%> 
        constrained="true" 		                        <%-- Options: true|false --%> 
        <%-- fetch="join|select" --%>                   <%-- Options: join|select --%> 
        <%-- property-ref=""  --%>                      <%-- The property-ref attribute should only be used for mapping legacy data where a foreign key refers to a unique key --%>
        												<%-- of the associated table other than the primary key. This is an ugly relational model. --%>
        access="property"  								<%-- Options: field|property|nosetter|ClassName --%>     
	/>
<%}%>

<%script type="core.AssociationEnd" name="generateOneToMany"%>
<%if ((isNavigable && upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
	<set name="<%associationPropertyName%>">
	    <key column="<%targetAssociationEnd.participant.identifierColumnName%>"/>
	    <one-to-many
	      class="<%targetAssociationEnd.participant.qualifiedName%>, 
		      	 <%targetAssociationEnd.participant.namespace.getRootContainer().name%>DomainModel"/>
	</set>
<%}%>

<%script type="core.AssociationEnd" name="generateManyToOne"%>
<%if ((isNavigable && upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
	<many-to-one
	  name="<%associationPropertyName%>"
	  column="<%targetAssociationEnd.participant.foreignKeyName(targetAssociationEnd.name)%>"
	  class="<%targetAssociationEnd.participant.qualifiedName%>, 
	  	<%targetAssociationEnd.participant.namespace.getRootContainer().name%>DomainModel"
	  not-null="false" 
	  cascade="<%cascadeMode%>"			<%-- Options: all|none|save-update|delete --%> 
      <%-- fetch="join|select" --%>                   <%-- Options: join|select --%> 
      <%-- update="true|false" --%>                   <%-- Options: true|false --%> 
      <%-- insert="true|false" --%>                   <%-- Options: true|false --%> 
      <%-- property-ref=""  --%>                      <%-- The property-ref attribute should only be used for mapping legacy data where a foreign key refers to a unique key --%>
	      												<%-- of the associated table other than the primary key. This is an ugly relational model. --%>
      access="property"  								<%-- Options: field|property|nosetter|ClassName --%>     
      <%-- unique="true|false" --%>                   <%-- Options: true|false --%> 
	/>
<%}%>

<%script type="core.AssociationEnd" name="generateManyToMany"%>
<%if ((isNavigable && upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
<%--	<many-to-many--%>
<%--        column="column_name"--%>
<%--        class="ClassName" --%>
<%--		fetch="join|select" --%>
<%--    /> --%>
TO DO: many-to-many
<%}%>

<%--------------------------------------------------------%>
<%--                 AUXILARY OPERATIONS                --%>
<%--------------------------------------------------------%>
<%script type="core.AssociationEnd" name="cascadeMode"%>
<%if (isComposite){%>all<%}else{%>none<%}%>

<%script type="core.AssociationEnd" name="inverseMode"%>
<%if (isNavigable){%>false<%}else{%>true<%}%>

<%script type="core.AssociationEnd" name="lazyMode"%>
<%if (targetAssociationEnd().participant.namespace != participant.namespace){%>true<%}else{%>false<%}%>

<%script type="core.Class" name="identifierColumnName"%>
<%if (thisContainsAnIdentifier()){%><%getIdentifier().name%><%}else{%>id<%}%>

<%script type="core.Classifier" name="qualifiedName"%>
<%getRootContainer().name%>.DomainModel.<%XLevelAndDownClassPath(2,".")%>.<%name%>

<%script type="core.Class" name="qualifiedNamespaceName"%>
<%getRootContainer().name%>.DomainModel.<%XLevelAndDownClassPath(2,".")%>

