<%--
  Author     Pascal Lindelauf
  E-Mail     plindelauf@yellowtail.nl
  Copyright  2007 Yellowtail
  All rights reserved.   This program and the accompanying materials
  are made available under the terms of the GNU Public License v2.0 or any later version
  which accompanies this distribution, and is available at
  http://www.gnu.org/licenses/gpl.txt
    You can apply any license to the files generated with this template
  and Acceleo.
--%>
<%--
  Target project: DomainModelPersistenceImpl
  This template generates a basic implementation of the simple repository interface class 
  based on NHibernate v1.2.0.
--%>
<%
metamodel http://www.obeo.fr/acceleo/uml14
%>

<%script type="uml14.Model" name="repository" file="<%name%>/DomainPersistenceImpl/Common/Repository.cs"%>
using System;
using System.Configuration;
using <%name%>.DomainModel.Common;
using NHibernate;
using NHibernate.Cfg;

namespace <%name%>.DomainPersistenceImpl.Common
{   
    public class Repository: IRepository
    {
        private ISession session;

        public Repository(ISession newSession)
        {
            this.session = newSession;
        }

        public ISession Session
        {
            get { return this.session; }
        }

        public bool IsOpen
        {
            get { 
                if (session == null || !session.IsOpen)
                {
                    return false;
                }

                // Session is open, connection could be either open or closed.
                // If connection is closed, signal that session is closed.
                if (!session.IsConnected)
                {
                    return false;
                }

                return true;
            }
        }

        public Object Get(Type entityType, object id)
        {
            if (!this.IsOpen)
            {
                throw new InvalidOperationException("Repository must be open before an entity can be retrieved.");
            }

            Object entity = (Object)this.session.Get(entityType, id);

            return entity;
        }

        public void SaveOrUpdate(Object entity)
        {
            if (!this.IsOpen)
            {
                throw new InvalidOperationException("Repository must be open before an entity can be retrieved.");
            }

            this.session.SaveOrUpdate(entity);
        }

        public void Delete(Object entity)
        {
            if (!this.IsOpen)
            {
                throw new InvalidOperationException("Repository must be open before an entity can be deleted.");
            }

            this.session.Delete(entity);
        }

        public void Persist()
        {
            if (!this.IsOpen)
            {
                throw new InvalidOperationException("Repository must be open before an entity can be retrieved.");
            }

            this.session.Flush();
        }

    }
}

