<%--
  Author     Pascal Lindelauf
  E-Mail     plindelauf@yellowtail.nl
  Copyright  2007 Yellowtail
  All rights reserved.   This program and the accompanying materials
  are made available under the terms of the GNU Public License v2.0 or any later version
  which accompanies this distribution, and is available at
  http://www.gnu.org/licenses/gpl.txt
--%>
<%--
  Target project: -
  This template generates a DDL script that creates a database structure based on the
  UML domain model. It applies different transformation rules for different classes.
  It generates tables, primary key constraints and foreign key constraints. It 
  also generates drop statements so that if a previous version of the database exists, 
  it will first be cleared.
--%>
<%
metamodel http://www.obeo.fr/acceleo/uml14

import com.yellowtail.uml14.gen.csharp.services.ClassService
import com.yellowtail.uml14.gen.csharp.services.AttributeService
import com.yellowtail.uml14.gen.csharp.services.AssociationService
import com.yellowtail.uml14.gen.csharp.services.Uml14Service
import com.yellowtail.uml14.gen.csharp.services.StringService
import com.yellowtail.uml14.gen.csharp.services.Uml14TypeResolver
import com.yellowtail.uml14.gen.csharp.utils.config
%>

<%script type="uml14.Model" name="domain-class-tables.sql" file="<%name%>/Resource/DBGeneration.sql"%>
<%-- TO DO: INTRA VS INTERMODULE, DATABASE NAME IN PROPERTY, INHERITANCE, DROP STATEMENTS OPTIONAL, MULTIPLICITY IN THE ATTRIBUTES --%>

<%-- Load the database name from config.mt --%>
USE [<%dbName%>];

<%-- GENERATE DROP TABLE STATEMENTS --%>
<%-- DROP FOREIGN KEY CONSTRAINTS --%>
<%for (eAllContents("Association")){%>
<%if (associatedClassesAreEntities()){%>
<%------- NOT CHECKED YET ---------------%>
<%generateXToOneDropFKConstraint%><%generateXToManyDropFKConstraint%><%generateOneToOneDropFKConstraint%><%generateOneToManyDropFKConstraint%><%generateManyToOneDropFKConstraint%><%generateManyToManyDropFKConstraint%>
<%------- NOT CHECKED YET ---------------%>
<%}%><%}%>

<%-- DROP FOREIGN KEY CONSTRAINTS FOR LINK TABLES --%>
<%for (eAllContents("Association")){%>
<%if (associatedClassesAreEntities()){%>
<%------- NOT CHECKED YET ---------------%>
<%generateXToOneDropLinkTable%><%generateXToManyDropLinkTable%><%generateOneToOneDropLinkTable%><%generateOneToManyDropLinkTable%><%generateManyToOneDropLinkTable%><%generateManyToManyDropLinkTable%>
<%------- NOT CHECKED YET ---------------%>
<%}%><%}%>

<%-- DROP REGULAR TABLES --%>
<%for (eAllContents("Class")){%>
<%if (hasStereotype("entityObject")){%>
IF EXISTS (SELECT * FROM SYSOBJECTS WHERE NAME = '<%tableName()%>')
DROP TABLE <%tableName()%>; 
GO
<%}%><%}%>

<%-- DROP LINK TABLES --%>
<%for (eAllContents("Association")){%>
<%if (associatedClassesAreEntities){%>
<%generateXToOneDropLinkTable%><%generateXToManyDropLinkTable%><%generateOneToOneDropLinkTable%><%generateOneToManyDropLinkTable%><%generateManyToOneDropLinkTable%><%generateManyToManyDropLinkTable%>
<%}%><%}%>

<%-- GENERATE CREATE TABLE STATEMENTS --%>
<%for (eAllContents("Class")){%>
<%if (hasStereotype("entityObject")){%>
CREATE TABLE <%tableName()%> ( 
<%-- BASIC ATTRIBUTES FIRST --%>
<%if (!thisContainsAnIdentifier){%>
	[id] int IDENTITY(1,1) NOT NULL<%if (attribute.nSize()>0){%>,<%}%>
<%}%>
<%for (attribute){%>
	<%-- If the iterator is beyond the first attribute, then make sure a seperating comma is generated --%>
	<%if (i>0){%>,<%}%>
	<%if (type.hasStereotype("valueObject")){%>
	<%-- If this attribute is of a valueObject type, generate the columns for the value object's attributes --%>
	<%generateXToOneValueObjectColumns(name)%>
	<%}else{%>
	<%if (type.hasStereotype("enumeration")){%>
	<%-- If this attribute is not of a valueObject type, generate the primitive column --%>
	[<%columnName()%>] int DEFAULT NULL
	<%}else{%>
	<%-- If this attribute is not of a valueObject type, generate the primitive column --%>
	<%if (isIdentifier){%>
	[<%columnName()%>] int IDENTITY(1,1) NOT NULL
	<%}else{%>
	[<%columnName()%>] <%type.resolveToSQLServer()%> DEFAULT NULL
	<%}%>
	<%}%>
	<%}%>
<%}%>
<%-- THEN THE ASSOCIATIONS --%>
<%for (association){%>
<%generateXToOneColumn%><%generateXToManyColumn%><%generateOneToOneColumn%><%generateOneToManyColumn%><%generateManyToOneColumn%><%generateManyToManyColumn%>
<%}%>
); 
<%}%><%}%>

<%-- CREATE LINK TABLES --%>
<%for (eAllContents("Association")){%>
<%if (associatedClassesAreEntities){%>
<%generateXToOneLinkTable%><%generateXToManyLinkTable%><%generateOneToOneLinkTable%><%generateOneToManyLinkTable%><%generateManyToOneLinkTable%><%generateManyToManyLinkTable%>
<%}%><%}%>

<%-- CREATE PRIMARY KEY CONSTRAINTS --%>
<%for (eAllContents("Class")){%>
<%if (hasStereotype("entityObject")){%>
ALTER TABLE <%tableName()%> ADD CONSTRAINT
<%if (!thisContainsAnIdentifier){%>
	PK_<%tableName()%>Id PRIMARY KEY (id)
<%}%>
<%for (attribute){%>
	<%if (isIdentifier){%>
	PK_<%owner.tableName()%><%columnName().firstUpper()%> PRIMARY KEY (<%columnName()%>)
	<%}%>
<%}%>
;
<%}%><%}%>

<%-- CREATE FOREIGN KEY CONSTRAINTS --%>
<%for (eAllContents("Association")){%>
<%if (associatedClassesAreEntities){%>
<%generateXToOneCreateFKConstraint%><%generateXToManyCreateFKConstraint%><%generateOneToOneCreateFKConstraint%><%generateOneToManyCreateFKConstraint%><%generateManyToOneCreateFKConstraint%><%generateManyToManyCreateFKConstraint%>
<%}%><%}%>

GO

<%--------------------------------------------------------%>
<%--                VALUE OBJECT HANDLING               --%>
<%--------------------------------------------------------%>
<%script type="core.Attribute" name="generateXToOneValueObjectColumns"%>
<%for (type.attribute){%>
	<%if (!type.hasStereotype("valueObject")){%>
	<%-- If this attribute is not of a valueObject type, generate the primitive column --%>
	<%-- If the iterator is beyond the first attribute, then make sure a seperating comma is generated --%>
	<%if (i>0){%>,<%}%>
	<%if (type.hasStereotype("enumeration")){%>
	<%-- If this attribute is not of a valueObject type, generate the primitive column --%>
	[<%args(0).firstLower()%><%columnName().firstUpper()%>] int DEFAULT NULL
	<%}else{%>
	[<%args(0).firstLower()%><%columnName().firstUpper()%>] <%type.resolveToSQLServer()%> DEFAULT NULL
	<%}%>
	<%}else{%>
	<%-- If this attribute is of a valueObject type, generate the columns for the value object's attributes --%>
	<%generateXToOneValueObjectColumns(args(0))%>
	<%}%>
<%}%>

<%--------------------------------------------------------%>
<%--    <unspecified> TO ONE RELATIONSHIP GENERATORS    --%>
<%--------------------------------------------------------%>
<%script type="core.AssociationEnd" name="generateXToOneColumn"%>
<%if (!isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1){%>
<%-- We assume there's always at least a PK column proceeding this, so the comma is okay --%>
	,[<%targetAssociationEnd.participant.foreignKeyName(targetAssociationEnd.name)%>] <%targetAssociationEnd.participant.foreignKeyTypeName%> <%targetAssociationEnd.nullableAssociationIndicator%>
<%}%>
<%script type="core.Association" name="generateXToOneLinkTable"%>
<%if (!sourceAssociationEnd.isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1){%>
<%-- No link table needed --%>
<%}%>
<%script type="core.Association" name="generateXToOneDropLinkTable"%>
<%if (!sourceAssociationEnd.isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1){%>
<%-- No link table present --%>
<%}%>
<%script type="core.Association" name="generateXToOneCreateFKConstraint"%>
<%if (!sourceAssociationEnd.isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1){%>
<%sourceAssociationEnd.createFKStatementForAssociationEnd%>
<%}%>
<%script type="core.Association" name="generateXToOneDropFKConstraint"%>
<%if (!sourceAssociationEnd.isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1){%>
<%sourceAssociationEnd.dropFKStatementForAssociationEnd%>
<%}%>

<%--------------------------------------------------------%>
<%--    <unspecified> TO MANY RELATIONSHIP GENERATORS   --%>
<%--------------------------------------------------------%>
<%script type="core.AssociationEnd" name="generateXToManyColumn"%>
<%if (!isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1){%>
<%-- The target table holds the reference --%>
<%}%>
<%script type="core.Association" name="generateXToManyLinkTable"%>
<%if (!sourceAssociationEnd.isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1){%>
<%-- No link table needed --%>
<%}%>
<%script type="core.Association" name="generateXToManyDropLinkTable"%>
<%if (!sourceAssociationEnd.isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1){%>
<%-- No link table present --%>
<%}%>
<%script type="core.Association" name="generateXToManyCreateFKConstraint"%>
<%if (!sourceAssociationEnd.isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1){%>
<%-- No FK constraint needed --%>
<%}%>
<%script type="core.Association" name="generateXToManyDropFKConstraint"%>
<%if (!sourceAssociationEnd.isNavigable && targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1){%>
<%-- No FK constraint present --%>
<%}%>

<%--------------------------------------------------------%>
<%--        ONE TO ONE RELATIONSHIP GENERATORS          --%>
<%--------------------------------------------------------%>
<%script type="core.AssociationEnd" name="generateOneToOneColumn"%>
<%if ((isNavigable && upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%-- We assume there's always at least a PK column proceeding this, so the comma is okay --%>
	,[<%targetAssociationEnd.participant.foreignKeyName(targetAssociationEnd.name)%>] <%targetAssociationEnd.participant.foreignKeyTypeName%> <%targetAssociationEnd.nullableAssociationIndicator%>
<%}%>
<%script type="core.Association" name="generateOneToOneLinkTable"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%-- No link table needed --%>
<%}%>
<%script type="core.Association" name="generateOneToOneDropLinkTable"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%-- No link table present --%>
<%}%>
<%script type="core.Association" name="generateOneToOneCreateFKConstraint"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%sourceAssociationEnd.createFKStatementForAssociationEnd%>
<%}%>
<%script type="core.Association" name="generateOneToOneDropFKConstraint"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%sourceAssociationEnd.dropFKStatementForAssociationEnd%>
<%}%>

<%--------------------------------------------------------%>
<%--        ONE TO MANY RELATIONSHIP GENERATORS         --%>
<%--------------------------------------------------------%>
<%script type="core.AssociationEnd" name="generateOneToManyColumn"%>
<%if ((isNavigable && upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
<%-- The target table holds the reference --%>
<%}%>
<%script type="core.Association" name="generateOneToManyLinkTable"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
<%-- No link table needed --%>
<%}%>
<%script type="core.Association" name="generateOneToManyDropLinkTable"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
<%-- No link table present --%>
<%}%>
<%script type="core.Association" name="generateOneToManyCreateFKConstraint"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
<%-- No FK constraint needed --%>
<%}%>
<%script type="core.Association" name="generateOneToManyDropFKConstraint"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
<%-- No FK constraint present --%>
<%}%>

<%--------------------------------------------------------%>
<%--        MANY TO ONE RELATIONSHIP GENERATORS         --%>
<%--------------------------------------------------------%>
<%script type="core.AssociationEnd" name="generateManyToOneColumn"%>
<%if ((isNavigable && upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%-- We assume there's always at least a PK column proceeding this, so the comma is okay --%>
	,[<%targetAssociationEnd.participant.foreignKeyName(targetAssociationEnd.name)%>] <%targetAssociationEnd.participant.foreignKeyTypeName%> <%targetAssociationEnd.nullableAssociationIndicator%>
<%}%>
<%script type="core.Association" name="generateManyToOneLinkTable"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%-- No link table needed --%>
<%}%>
<%script type="core.Association" name="generateManyToOneDropLinkTable"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%-- No link table present --%>
<%}%>
<%script type="core.Association" name="generateManyToOneCreateFKConstraint"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%sourceAssociationEnd.createFKStatementForAssociationEnd%>
<%}%>
<%script type="core.Association" name="generateManyToOneDropFKConstraint"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==1)){%>
<%sourceAssociationEnd.dropFKStatementForAssociationEnd%>
<%}%>

<%--------------------------------------------------------%>
<%--        MANY TO MANY RELATIONSHIP GENERATORS        --%>
<%--------------------------------------------------------%>
<%script type="core.AssociationEnd" name="generateManyToManyColumn"%>
<%if ((isNavigable && upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
<%-- A link table will be needed --%>
<%}%>
<%script type="core.Association" name="generateManyToManyLinkTable"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
CREATE TABLE <%linkTableName()%> ( 
	[<%sourceAssociationEnd.participant.foreignKeyName(sourceAssociationEnd.name)%>] <%sourceAssociationEnd.participant.foreignKeyTypeName%> NOT NULL,
	[<%targetAssociationEnd.participant.foreignKeyName(targetAssociationEnd.name)%>] <%targetAssociationEnd.participant.foreignKeyTypeName%> NOT NULL
); 
<%}%>
<%script type="core.Association" name="generateManyToManyDropLinkTable"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
IF EXISTS (SELECT * FROM SYSOBJECTS WHERE NAME = '<%linkTableName()%>')
DROP TABLE <%linkTableName()%>;
GO
<%}%>
<%script type="core.Association" name="generateManyToManyCreateFKConstraint"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
<%createFKStatementForLinkTableClass0%>
<%createFKStatementForLinkTableClass1%>
<%}%>
<%script type="core.Association" name="generateManyToManyDropFKConstraint"%>
<%if ((sourceAssociationEnd.isNavigable && sourceAssociationEnd.upperBound()==-1) && (targetAssociationEnd.isNavigable && targetAssociationEnd.upperBound()==-1)){%>
<%dropFKStatementForLinkTableClass0%>
<%dropFKStatementForLinkTableClass1%>
<%}%>

<%--------------------------------------------------------%>
<%--                 AUXILARY OPERATIONS                --%>
<%--------------------------------------------------------%>
<%script type="core.AssociationEnd" name="createFKStatementForAssociationEnd"%>
ALTER TABLE <%participant.tableName()%> ADD CONSTRAINT FK_<%participant.tableName()%>_<%targetAssociationEnd.targetAssociationEndFKConstraintTableName%>
FOREIGN KEY (<%targetAssociationEnd.participant.foreignKeyName(targetAssociationEnd.name)%>) REFERENCES <%targetAssociationEnd.participant.tableName()%> 
(<%if (targetAssociationEnd.participant.thisContainsAnIdentifier){%><%targetAssociationEnd.participant.getIdentifier().columnName()%><%}else{%>id<%}%>);

<%script type="core.Association" name="createFKStatementForLinkTableClass0"%>
ALTER TABLE <%linkTableName()%> ADD CONSTRAINT FK_<%linkTableName()%>_<%associatedClasses().nGet(0).tableName()%>
FOREIGN KEY (<%associatedClasses().nGet(0).foreignKeyName(associatedClasses().nGet(0).name)%>) REFERENCES <%associatedClasses().nGet(0).targetAssociationEndFKConstraintTableName%> 
(<%if (associatedClasses().nGet(0).thisContainsAnIdentifier){%><%associatedClasses().nGet(0).getIdentifier().columnName()%><%}else{%>id<%}%>);

<%script type="core.Association" name="createFKStatementForLinkTableClass1"%>
ALTER TABLE <%linkTableName()%> ADD CONSTRAINT FK_<%linkTableName()%>_<%associatedClasses().nGet(1).tableName()%>
FOREIGN KEY (<%associatedClasses().nGet(1).foreignKeyName(associatedClasses().nGet(1).name)%>) REFERENCES <%associatedClasses().nGet(1).targetAssociationEndFKConstraintTableName%> 
(<%if (associatedClasses().nGet(1).thisContainsAnIdentifier){%><%associatedClasses().nGet(1).getIdentifier().columnName()%><%}else{%>id<%}%>);

<%script type="core.AssociationEnd" name="dropFKStatementForAssociationEnd"%>
IF EXISTS (SELECT * FROM SYSOBJECTS WHERE NAME = '<%participant.tableName()%>')
IF EXISTS (SELECT * FROM SYSOBJECTS WHERE NAME = 'FK_<%participant.tableName()%>_<%targetAssociationEnd.targetAssociationEndFKConstraintTableName%>')
ALTER TABLE <%participant.tableName()%> DROP CONSTRAINT FK_<%participant.tableName()%>_<%targetAssociationEnd.targetAssociationEndFKConstraintTableName%>;
GO 

<%script type="core.Association" name="dropFKStatementForLinkTableClass0"%>
IF EXISTS (SELECT * FROM SYSOBJECTS WHERE NAME = '<%linkTableName()%>')
IF EXISTS (SELECT * FROM SYSOBJECTS WHERE NAME = 'FK_<%linkTableName()%>_<%associatedClasses().nGet(0).targetAssociationEndFKConstraintTableName%>')
ALTER TABLE <%linkTableName()%> DROP CONSTRAINT FK_<%linkTableName()%>_<%associatedClasses().nGet(0).targetAssociationEndFKConstraintTableName%>;
GO 

<%script type="core.Association" name="dropFKStatementForLinkTableClass1"%>
IF EXISTS (SELECT * FROM SYSOBJECTS WHERE NAME = '<%linkTableName()%>')
IF EXISTS (SELECT * FROM SYSOBJECTS WHERE NAME = 'FK_<%linkTableName()%>_<%associatedClasses().nGet(1).targetAssociationEndFKConstraintTableName%>')
ALTER TABLE <%linkTableName()%> DROP CONSTRAINT FK_<%linkTableName()%>_<%associatedClasses().nGet(1).targetAssociationEndFKConstraintTableName%>;
GO

<%script type="core.AssociationEnd" name="nullableAssociationIndicator"%>
<%if (targetAssociationEnd.lowerBound==0){%>DEFAULT NULL<%}else{%>NOT NULL<%}%>

<%script type="core.AssociationEnd" name="intraModule"%>
<%if (participant.namespace==targetAssociationEnd.participant.namespace){%>true<%}else{%>false<%}%>

<%script type="core.Association" name="intraModule"%>
<%if (associatedClasses().nGet(0).namespace==associatedClasses().nGet(1).namespace){%>true<%}else{%>false<%}%>

<%script type="core.Association" name="sourceAssociationEnd"%>
<%connection.nGet(0)%>

<%script type="core.Association" name="targetAssociationEnd"%>
<%connection.nGet(1)%>

<%script type="core.AssociationEnd" name="targetAssociationEndFKConstraintTableName"%>
<%if (name==""){%><%participant.tableName()%><%}else{%><%name.firstUpper()%><%}%>