/*
 * Author     Pascal Lindelauf
 * E-Mail     plindelauf@yellowtail.nl
 * Copyright  2007 Yellowtail
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the GNU Public License v2.0 or any later version
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.txt
 */
/*
 * This class contains operations that simplify access to  
 * UML assocation elements.
 */
package com.yellowtail.uml14.gen.csharp.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;

import fr.obeo.acceleo.ecore.factories.FactoryException;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.uml14.foundation.core.Association;
import fr.obeo.acceleo.uml14.foundation.core.AssociationEnd;
import fr.obeo.acceleo.uml14.foundation.core.Classifier;
import fr.obeo.acceleo.uml14.foundation.data_types.AggregationKind;
import fr.obeo.acceleo.uml14.foundation.data_types.MultiplicityRange;

/**
 * @author pli
 *
 */

public class AssociationService {	
	private StringService stringService = new StringService();
	private Uml14Service uml14Service = new Uml14Service();

	public AssociationEnd targetAssociationEnd(AssociationEnd sourceAssociationEnd) throws ENodeCastException {
		EList allAssociationEnds = sourceAssociationEnd.getAssociation().getConnection();
		AssociationEnd associationEnd = null;
		Boolean foundOtherEnd = Boolean.FALSE;
		
		for (Iterator i = allAssociationEnds.iterator(); i.hasNext() && !foundOtherEnd.booleanValue(); ){
			associationEnd = (AssociationEnd)i.next();
			foundOtherEnd = new Boolean(!associationEnd.equals(sourceAssociationEnd));
		}
		return (foundOtherEnd.booleanValue() ? associationEnd : null);
	}

	public Collection associatedClasses(Association association) throws ENodeCastException {
		Collection classes = new ArrayList();
		classes.add(((AssociationEnd)association.getConnection().get(0)).getParticipant());
		classes.add(((AssociationEnd)association.getConnection().get(1)).getParticipant());
		return classes;
	}

	public boolean associatedClassesAreEntities(Association association) throws ENodeCastException, FactoryException {
		Collection classes = associatedClasses(association);
		boolean areEntities = false;
		
		for (Iterator i = classes.iterator(); i.hasNext() && !areEntities; ){
			areEntities = uml14Service.hasStereotype((Classifier)i.next(), "entityObject");
		}
		return areEntities;
	}

	public String linkTableName(Association association) throws ENodeCastException {
		Collection classes = associatedClasses(association);
		String tableName = "tbl";
		
		for (Iterator i = classes.iterator(); i.hasNext(); ){
			tableName += stringService.firstUpper(((Classifier)i.next()).getName());
		}
		
		return tableName; 
	}

	public boolean isIntraModuleAssociation(Association assoc) throws ENodeCastException {
		return (assoc.getNamespace() == assoc.getNamespace());
	}

	public boolean isInterModuleAssociation(Association assoc) throws ENodeCastException {
		return (assoc.getNamespace() != assoc.getNamespace());
	}

	public int upperBound(AssociationEnd associationEnd) throws ENodeCastException {
		return (int)((MultiplicityRange)associationEnd.getMultiplicity().getRange().get(0)).getUpper();
	}

	public int lowerBound(AssociationEnd associationEnd) throws ENodeCastException {
		return (int)((MultiplicityRange)associationEnd.getMultiplicity().getRange().get(0)).getLower();
	}

	public boolean isZeroToOneAssociation(AssociationEnd associationEnd) throws ENodeCastException {
		return lowerBound(associationEnd) == 0 && upperBound(associationEnd) == 1;
	}

	public boolean isOneToOneAssociation(AssociationEnd associationEnd) throws ENodeCastException {
		return lowerBound(associationEnd) == 1 && upperBound(associationEnd) == 1;
	}
	
	public boolean isZeroToManyAssociation(AssociationEnd associationEnd) throws ENodeCastException {
		return lowerBound(associationEnd) == 0 && upperBound(associationEnd) == -1;
	}
	
	public boolean isOneToManyAssociation(AssociationEnd associationEnd) throws ENodeCastException {
		return lowerBound(associationEnd) == 1 && upperBound(associationEnd) == -1;
	}

	public boolean isComposite(AssociationEnd associationEnd) throws ENodeCastException {
		return associationEnd.getAggregation().getValue() == AggregationKind.AK_COMPOSITE;
	}

	public String associationPropertyName(AssociationEnd associationEnd) throws ENodeCastException {
		if (targetAssociationEnd(associationEnd).getName() == null){
			return stringService.firstUpper(targetAssociationEnd(associationEnd).getParticipant().getName());
		}else{
			return stringService.firstUpper(targetAssociationEnd(associationEnd).getName());
		}	
	}
}