/*
 * Author     Pascal Lindelauf
 * E-Mail     plindelauf@yellowtail.nl
 * Copyright  2007 Yellowtail
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the GNU Public License v2.0 or any later version
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.txt
 */
/*
 * This class contains operations that simplify access to  
 * UML attribute elements.
 */
package com.yellowtail.uml14.gen.csharp.services;

import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.uml14.foundation.core.Attribute;
import fr.obeo.acceleo.uml14.foundation.data_types.MultiplicityRange;

/**
 * @author pli
 *
 */

public class AttributeService {
	
	public static final String IDENTIFIER_TYPE_NAME = "ObjectId";
	private StringService stringService = new StringService();
	
	public String columnName(Attribute att) throws ENodeCastException {
		return stringService.firstLower(att.getName());
	}

	public String attributeName(Attribute att) throws ENodeCastException {
		return stringService.firstLower(att.getName());
	}

	public String propertyName(Attribute att) throws ENodeCastException {
		return stringService.firstUpper(att.getName());
	}

	public boolean isIdentifier(Attribute att) throws ENodeCastException {
		if (att.getType() == null){
			return false;
		}else{
			return att.getType().getName().equalsIgnoreCase(IDENTIFIER_TYPE_NAME);
		}
	}
	
	public int attributeUpperBound(Attribute att) throws ENodeCastException {
		return (int)((MultiplicityRange)att.getMultiplicity().getRange().get(0)).getUpper();
	}

	public int attributeLowerBound(Attribute att) throws ENodeCastException {
		return (int)((MultiplicityRange)att.getMultiplicity().getRange().get(0)).getLower();
	}

	public boolean isZeroToOneAttribute(Attribute att) throws ENodeCastException {
		return attributeLowerBound(att) == 0 && attributeUpperBound(att) == 1;
	}

	public boolean isOneToOneAttribute(Attribute att) throws ENodeCastException {
		return attributeLowerBound(att) == 1 && attributeUpperBound(att) == 1;
	}
	
	public boolean isZeroToManyAttribute(Attribute att) throws ENodeCastException {
		return attributeLowerBound(att) == 0 && attributeUpperBound(att) == -1;
	}
	
	public boolean isOneToManyAttribute(Attribute att) throws ENodeCastException {
		return attributeLowerBound(att) == 1 && attributeUpperBound(att) == -1;
	}

}