/*
 * Author     Pascal Lindelauf
 * E-Mail     plindelauf@yellowtail.nl
 * Copyright  2007 Yellowtail
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the GNU Public License v2.0 or any later version
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.txt
 */
/*
 * This class contains operations that resolve primitive types   
 * to the different target technologies (C# and SQL Server).
 */
package com.yellowtail.uml14.gen.csharp.services;

import java.util.HashMap;
import java.util.Map;

import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.uml14.foundation.core.Classifier;
 
/**
 * @author pli
 *
 */

public class Uml14TypeResolver {
	private Map uml14ToCSharpMap;
	private Map uml14ToSQLServerMap;
	
	public Uml14TypeResolver() {
		// Here to resolve primitive types
		uml14ToCSharpMap = new HashMap();
		uml14ToSQLServerMap = new HashMap();
				
		uml14ToCSharpMap.put("string", "string");
		uml14ToCSharpMap.put("integer", "int");
		uml14ToCSharpMap.put("double", "double");	
		uml14ToCSharpMap.put("decimal", "decimal");	
		uml14ToCSharpMap.put("bool", "bool");		
		uml14ToCSharpMap.put("objectid", "int");		
		uml14ToCSharpMap.put("date", "DateTime");	
		uml14ToCSharpMap.put("datetime", "DateTime");	
		uml14ToCSharpMap.put("percentage", "decimal");	
		uml14ToCSharpMap.put("", "void");	

		uml14ToSQLServerMap.put("string", "varchar(255)");
		uml14ToSQLServerMap.put("integer", "int");
		uml14ToSQLServerMap.put("double", "decimal");	
		uml14ToSQLServerMap.put("decimal", "decimal(18,2)");	
		uml14ToSQLServerMap.put("bool", "bit");		
		uml14ToSQLServerMap.put("objectid", "int");		
		uml14ToSQLServerMap.put("date", "datetime");		
		uml14ToSQLServerMap.put("datetime", "datetime");		
		uml14ToSQLServerMap.put("percentage", "decimal(5,2)");		
		uml14ToSQLServerMap.put("", "void");	
	}
	
	public String resolveToCSharp(Classifier type) throws ENodeCastException {
		if (type == null){
			return "";
		}else{
			String resolvedType = (String)uml14ToCSharpMap.get(type.getName().toLowerCase());
			return (resolvedType == null) ? type.getName() : resolvedType;
		}
	}

	public String resolveToSQLServer(Classifier type) throws ENodeCastException {
		if (type == null){
			return "";
		}else{
			String resolvedType = (String)uml14ToSQLServerMap.get(type.getName().toLowerCase());
			return (resolvedType == null) ? type.getName() : resolvedType;
		}
	}
}
