package fr.obeo.weblog.business.blog;

// Start of user code for import
import java.io.Serializable;
import java.util.Date;

// End of user code

/**
 * 
 */
public class BlogEntry implements Serializable {

	// serialVersionUID is used for serialization.
	private static final long serialVersionUID = 1L;

	public final static String PROP_ID = "id";
	public final static String PROP_TITLE = "title";
	public final static String PROP_CREATIONDATE = "creationDate";
	public final static String PROP_CONTENT = "content";
	public final static String PROP_AUTHORUSER = "authorUser";

	private String id;
	protected String title;
	protected Date creationDate;
	protected String content;
	protected User authorUser;

	/**
	 * Default constructor 
	 */
	public BlogEntry() {
		super();
	}
	
	/**
	 * Return the identifier
	 * @return Id
	 */
	public String getId() {
		return this.id;
	}

	/**
	 * Set a value to parameter Id
	 * @param id Value of the identifier
	 */
	public void setId(String id) {
		this.id = id;
	}
	

	/**
	 * Constructor with all parameters initialized
	 * @param title 
	 * @param creationDate 
	 * @param content 
	 * @param authorUser 
	 */
	public BlogEntry(String title, Date creationDate, String content, User authorUser) {
		this.title = title;
		this.creationDate = creationDate;
		this.content = content;
		this.authorUser = authorUser;
	}	

	/**
	 * Return title
	 * @return 
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * Set a value to parameter title.
	 * @param title 
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * Return creationDate
	 * @return 
	 */
	public Date getCreationDate() {
		return creationDate;
	}

	/**
	 * Set a value to parameter creationDate.
	 * @param creationDate 
	 */
	public void setCreationDate(Date creationDate) {
		this.creationDate = creationDate;
	}

	/**
	 * Return content
	 * @return 
	 */
	public String getContent() {
		return content;
	}

	/**
	 * Set a value to parameter content.
	 * @param content 
	 */
	public void setContent(String content) {
		this.content = content;
	}

	/**
	 * Return authorUser
	 * @return 
	 */
	public User getAuthorUser() {
		return authorUser;
	}

	/**
	 * Set a value to parameter authorUser.
	 * @param authorUser 
	 */
	public void setAuthorUser(User authorUser) {
		this.authorUser = authorUser;
	}


	
	/** 
	 * Equality test based attributes values
	 * @param value Value to compare
	 */
	public boolean equals(Object other) {
		if (this == other) {
			return true;
		}
		if ( ! (other instanceof BlogEntry)) {
			return false;
		}
		
		final BlogEntry castedOther = (BlogEntry) other;
		
		if ( id != null && castedOther.getId() != null ) {
			return id.equals( castedOther.getId() );
		}
		
		if ( (	title == null && castedOther.getTitle() != null) || 
			 (	title != null && !title.equals(castedOther.getTitle()) ) 
		   ) return false;
		if ( (	creationDate == null && castedOther.getCreationDate() != null) || 
			 (	creationDate != null && !creationDate.equals(castedOther.getCreationDate()) ) 
		   ) return false;
		if ( (	content == null && castedOther.getContent() != null) || 
			 (	content != null && !content.equals(castedOther.getContent()) ) 
		   ) return false;
		if ( (	authorUser == null && castedOther.getAuthorUser() != null) || 
			 (	authorUser != null && !authorUser.equals(castedOther.getAuthorUser()) ) 
		   ) return false;
		
		return true;
	}

	/** 
	 * HashTable code based on attributes hashcodes
	 * @return hash value
	 */
	public int hashCode() {
		int result = 0;
		if (id != null) result = id.hashCode();
		if (title != null) result = 29 * result + title.hashCode();
		if (creationDate != null) result = 29 * result + creationDate.hashCode();
		if (content != null) result = 29 * result + content.hashCode();
		return result;
	}
   
	// Start of user code for private methods
	// End of user code
   
}