package fr.obeo.weblog.dao;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import fr.obeo.fwk.dao.exception.DaoException;
import fr.obeo.weblog.dao.blog.api.IUserDao;
import fr.obeo.weblog.dao.blog.api.IBlogEntryDao;

public class WeblogDaoFactory {
	
	private final static String FILE_DAO_MAPPING = "dao";
	private static ResourceBundle bundle = ResourceBundle.getBundle(FILE_DAO_MAPPING);

	public static IUserDao getUserDao() throws DaoException {

		try {
			String className = bundle.getString(IUserDao.class.getName());

			// Cast and instanciate with JDK 1.5 control
			Class<? extends IUserDao> daoClass = 
				Class.forName(className).asSubclass(IUserDao.class);
			return daoClass.newInstance();

		} catch (MissingResourceException e) {
			throw new DaoException("Key " + IUserDao.class.getName() + " not found in " + FILE_DAO_MAPPING + ".properties", e);
		} catch (ClassNotFoundException e) {
			throw new DaoException("DAO implementation not found for " + IUserDao.class.getName(), e);
		} catch (InstantiationException e) {
			throw new DaoException(e);
		} catch (IllegalAccessException e) {
			throw new DaoException(e);
		}
		
	}
	
	public static IBlogEntryDao getBlogEntryDao() throws DaoException {

		try {
			String className = bundle.getString(IBlogEntryDao.class.getName());

			// Cast and instanciate with JDK 1.5 control
			Class<? extends IBlogEntryDao> daoClass = 
				Class.forName(className).asSubclass(IBlogEntryDao.class);
			return daoClass.newInstance();

		} catch (MissingResourceException e) {
			throw new DaoException("Key " + IBlogEntryDao.class.getName() + " not found in " + FILE_DAO_MAPPING + ".properties", e);
		} catch (ClassNotFoundException e) {
			throw new DaoException("DAO implementation not found for " + IBlogEntryDao.class.getName(), e);
		} catch (InstantiationException e) {
			throw new DaoException(e);
		} catch (IllegalAccessException e) {
			throw new DaoException(e);
		}
		
	}
	
}