package fr.obeo.weblog.dao.blog.hibernate;

// Start of user code for import
import java.util.Collection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;

import fr.obeo.fwk.dao.exception.DaoException;
import fr.obeo.fwk.dao.hibernate.HibernateUtil;
import fr.obeo.weblog.business.blog.BlogEntry;
import fr.obeo.weblog.dao.blog.api.IBlogEntryDao;
// End of user code for import

public class HibernateBlogEntryDao implements IBlogEntryDao {
	private final static Log log = LogFactory.getLog(HibernateBlogEntryDao.class);

	/**
	 * Create a new element
	 * @param blogEntry Element to create.
	 */
	public void createBlogEntry(BlogEntry blogEntry) throws DaoException {
		log.debug("Create a new BlogEntry entity");
	
		try {
			Session session = HibernateUtil.currentSession();
			session.save(blogEntry);

		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}

	/**
	 * Update an existing element
	 * @param blogEntry Element to update. If the element has an id, it may use it.
	 */
	public void updateBlogEntry(BlogEntry blogEntry) throws DaoException{
		log.debug("Update the entity BlogEntry with id =" + blogEntry.getId());
		
		try {
			Session session = HibernateUtil.currentSession();
			session.update(blogEntry);

		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}

	/**
	 * Delete an element
	 * @param blogEntry Element which will be delete. 
	 * Only id can be used to find which element must be deleted.
	 */
	public void deleteBlogEntry(BlogEntry blogEntry) throws DaoException {
		log.debug("Delete the entity BlogEntry with id =" + blogEntry.getId());
		
		try {
			Session session = HibernateUtil.currentSession();
			session.delete(blogEntry);

		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}

	/**
	 * Find all elements.
	 * @return A list with all elements, without any filter.
	 */
	public Collection<BlogEntry> findAllBlogEntrys() throws DaoException {
		log.debug("Find all instance of BlogEntry entity");
		
		try {
			Session session = HibernateUtil.currentSession();
			Criteria criteria = session.createCriteria(BlogEntry.class);
			Collection<BlogEntry> resultList = criteria.list();
			
			log.debug("Find " + resultList.size() + " instances of BlogEntry entity");
			return resultList;
			
		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}

	/**
	 * Find one entity by its primary key.
	 * @return The entity found.
	 */
	public BlogEntry findBlogEntryById(String id) throws DaoException {
		log.debug("Find one instance of BlogEntry entity by id : " + id);
		
		try {
			Session session = HibernateUtil.currentSession();
			Criteria criteria = session.createCriteria(BlogEntry.class)
				.add(Restrictions.eq("id", id));
			BlogEntry result = (BlogEntry)criteria.uniqueResult();
			
			return result;
			
		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}
	


	//Start of user code
	//End of user code
}