package fr.obeo.weblog.dao.blog.hibernate;

// Start of user code for import
import java.util.Collection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;

import fr.obeo.fwk.dao.exception.DaoException;
import fr.obeo.fwk.dao.hibernate.HibernateUtil;
import fr.obeo.weblog.business.blog.User;
import fr.obeo.weblog.dao.blog.api.IUserDao;
// End of user code for import

public class HibernateUserDao implements IUserDao {
	private final static Log log = LogFactory.getLog(HibernateUserDao.class);

	/**
	 * Create a new element
	 * @param user Element to create.
	 */
	public void createUser(User user) throws DaoException {
		log.debug("Create a new User entity");
	
		try {
			Session session = HibernateUtil.currentSession();
			session.save(user);

		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}

	/**
	 * Update an existing element
	 * @param user Element to update. If the element has an id, it may use it.
	 */
	public void updateUser(User user) throws DaoException{
		log.debug("Update the entity User with id =" + user.getId());
		
		try {
			Session session = HibernateUtil.currentSession();
			session.update(user);

		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}

	/**
	 * Delete an element
	 * @param user Element which will be delete. 
	 * Only id can be used to find which element must be deleted.
	 */
	public void deleteUser(User user) throws DaoException {
		log.debug("Delete the entity User with id =" + user.getId());
		
		try {
			Session session = HibernateUtil.currentSession();
			session.delete(user);

		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}

	/**
	 * Find all elements.
	 * @return A list with all elements, without any filter.
	 */
	public Collection<User> findAllUsers() throws DaoException {
		log.debug("Find all instance of User entity");
		
		try {
			Session session = HibernateUtil.currentSession();
			Criteria criteria = session.createCriteria(User.class);
			Collection<User> resultList = criteria.list();
			
			log.debug("Find " + resultList.size() + " instances of User entity");
			return resultList;
			
		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}

	/**
	 * Find one entity by its primary key.
	 * @return The entity found.
	 */
	public User findUserById(String id) throws DaoException {
		log.debug("Find one instance of User entity by id : " + id);
		
		try {
			Session session = HibernateUtil.currentSession();
			Criteria criteria = session.createCriteria(User.class)
				.add(Restrictions.eq("id", id));
			User result = (User)criteria.uniqueResult();
			
			return result;
			
		} catch (HibernateException e) {
			throw new DaoException(e);
		}
	}
	

		/**
	 * Get a User according to its login and password
	 * @param login 
	 * @param password 
	 * @return 
	 */
	public User findByLoginAndPassword(String login, String password) throws DaoException {		
		// Start of user code for findByLoginAndPassword
		log.debug("Call findByLoginAndPassword on User entity");		
		try {
			Session session = HibernateUtil.currentSession();
			Criteria criteria = session.createCriteria(User.class)
				.add(Restrictions.eq("login", login))
				.add(Restrictions.eq("password", password));

			User result = (User)criteria.uniqueResult();
			return result;

		} catch (HibernateException e) {
			throw new DaoException(e);
		}
		// End of user code for findByLoginAndPassword
	}

	//Start of user code
	//End of user code
}