package fr.obeo.weblog.ui;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Transaction;

import fr.obeo.fwk.dao.hibernate.HibernateUtil;

public class HibernateFilter implements Filter {
    /**
     * Commons Logging instance.
     */
    private static Log log = LogFactory.getLog(HibernateFilter.class);

    public void doFilter(ServletRequest request,
                         ServletResponse response,
                         FilterChain chain)
            throws IOException, ServletException {

    	Transaction tx = null;
        try {
            log.debug("Starting a database transaction");
            tx = HibernateUtil.currentSession().beginTransaction();

            // Call the next filter (continue request processing)
            chain.doFilter(request, response);

            // Commit and cleanup
            log.debug("Committing the database transaction");
            tx.commit();

        } catch (Throwable ex) {
            // Rollback only
            if (tx != null) {
            	try {
	                log.debug("Trying to rollback database transaction after exception");
	                tx.rollback();
	            } catch (Throwable rbEx) {
	                log.error("Could not rollback transaction after exception!", rbEx);
	            }
        }

            // Let others handle it... maybe another interceptor for exceptions ?
            throw new ServletException(ex);
        }
    }

    public void init(FilterConfig filterConfig) throws ServletException {}
    public void destroy() {}

}