/**
 * Copyright © 2007 - Obeo
 * All rights reserved.  This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 1.0
 * 
 * You can apply any license to the files generated with this template
 * and Acceleo.
 *
 * @author  www.obeo.fr
 * 
 **/package org.acceleo.module.pim.uml21.services;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;


/**
 * Acceleo services for String.
 *
 */
public class StringServices {
	
//	public Strings() {
//		Trace.TRACE = true;
//	}
	
	/**
	 * Return the String with the first letter uppercase. Useful for class name.
	 */
	public String firstUpper(String s) throws ENodeCastException {
		if (s.length() > 0) 
			s = Character.toUpperCase(s.charAt(0)) + s.substring(1);
		return s;
	}

	/**
	 * Return the String with the first letter lowercase. Useful for attribute name.
	 */
	public String firstLower(String s) throws ENodeCastException {
		if (s.length() > 0) 
			s = Character.toLowerCase(s.charAt(0)) + s.substring(1);
		return s;
	}

	/**
	 * Compare to strings.
	 */
	public int compare(ENode node, String s1, String s2) throws ENodeCastException {
		return s1.compareTo(s2);
	}

	/**
	 * Replace all instance of one String by another.
	 * @param text Text to tranform
	 * @param arg1 Text to search
	 * @param arg2 Text used for remplace
	 */
	public String replaceAll(String text, String arg1, String arg2) throws ENodeCastException {
		return text.replaceAll(arg1,arg2);
	}
	
	/**
	 * Transform a String with notation aa.bb.cc to aa/bb/cc<br/>
	 * Usefull for package translating.
	 */
	public String toPath(String packageName) throws ENodeCastException {
		return packageName.trim().replaceAll("\\.","/");
	}

	/**
	 * Return a unique code from a String.
	 */
	public int uniqueCode(String s){
		return s.hashCode();
	}

	/**
	 * Convert to standard notation.<br/>
	 * Remove spaces, non letter/digit charaters.
	 * @param name Name to tranform 
	 * @return The String normalized
	 */
	public String toStandardNotation(String name){
		StringBuffer dest = new StringBuffer(name.length());
		
		boolean nextMaj = false;
		for (int i = 0; i < name.length(); i++) {
			char c = name.charAt(i);
			
			if(!Character.isLetterOrDigit(c) && !Character.isSpaceChar(c)) {
				// ignore
				continue;
			}
			
			switch (c) {
			case ' ':
				nextMaj = true;break;
			case 'é':
			case 'è':
			case 'ê':
				dest.append('e');break;
			case 'à':
				dest.append('a');break;
			case 'ù':
				dest.append('u');break;
			default:
				if (nextMaj) {
					nextMaj = false;
					dest.append(Character.toUpperCase(c));					
				} else {
					dest.append(c);
				} break;
			}
		}
		return dest.toString();
	}

	/**
	 * Convert to standard package notation.<br/>
	 * Remove spaces, non letter/digit charaters and transfomr to lower case.
	 * @param name Name to tranform 
	 * @return The String normalized
	 */
	public String toPkgName(String name){		
		return toStandardNotation(name).toLowerCase();
	}
	
	/**
	 * Repeat nb times the s String.
	 * @param node Just for apply service.
	 * @param s The String to repeat.
	 * @param nb How many time it must be repeated.
	 * @return
	 */
	public String repeat(ENode node, String s, int nb) {
		int size = nb * s.length();
		StringBuffer strBuf = new StringBuffer(size);
		for (int i = 0; i < size; i++) {
			strBuf.append(s);
		}
		return strBuf.toString();
	}
	
}
