/**
 * Copyright  2007 - Obeo
 * All rights reserved.  This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 1.0
 * 
 * You can apply any license to the files generated with this template
 * and Acceleo.
 *
 * @author  www.obeo.fr
 * 
 **/
package org.acceleo.module.pim.uml21.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ParameterDirectionKind;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;

import fr.obeo.acceleo.ecore.factories.FactoryException;
import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeIterator;
import fr.obeo.acceleo.gen.template.eval.ENodeList;


public class Uml2Services {

	/**
	 * Verify if an Element have a stereotype.
	 * Use keyword and profil to find stereotype. 
	 * Multiple stereotype are allow.
	 * @param elt Element used.
	 * @param stereotype Stereotype to search.
	 * @return true if found. false else.
	 * @throws ENodeCastException
	 */
	public boolean hasStereotype(Element elt, String stereotype) throws ENodeCastException {
		Stereotype stereotypeFound = elt.getAppliedStereotype(stereotype);
		if (stereotypeFound == null) {
			
			// search with keywords
			if (elt.hasKeyword(stereotype))
					return true;
			
			return false;
		}
		else
			return true;		
	}	
	
	/**
	 * Filter a list of Element which have to stereotype specify as parameter
	 * @param nodes
	 * @param stereotype
	 * @throws ENodeCastException
	 */
	public ENodeList filterWithStereotype(ENodeList nodes, String stereotype) throws ENodeCastException {
		ENodeList result = new ENodeList();
		ENodeIterator it = nodes.iterator();
		while (it.hasNext()){
			ENode node = it.next();
			if (node.isEObject() && node.getEObject() instanceof Element){
				Element elt = (Element)node.getEObject();
				if (hasStereotype(elt, stereotype)) {
					result.add(node);
					//System.out.println("With : "+((NamedElement)elt).getName());
				}
			}
		}
		return result;
	}
	
	//TODO :  enlever aprs correction bug 58 et  laisser dans .mt
	public ENodeList attributeIds(ENodeList nodes) throws ENodeCastException {
		return filterWithStereotype(nodes, "id");
	}
	
	
	/**
	 * Filter a list of Element which doesn't have to stereotype specify as parameter
	 * @param nodes
	 * @param stereotype
	 * @throws ENodeCastException
	 */
	public ENodeList filterWithOutStereotype(ENodeList nodes, String stereotype) throws ENodeCastException {
		ENodeList result = new ENodeList();
		ENodeIterator it = nodes.iterator();
		while (it.hasNext()){
			ENode node = it.next();
			if (node.isEObject() && node.getEObject() instanceof Element){
				Element elt = (Element)node.getEObject();
				if (!hasStereotype(elt, stereotype)) {
					result.add(node);
					//System.out.println("Without : "+((NamedElement)elt).getName());
				}
			}
		}
		return result;
	}

	public ENodeList attributeNotIds(ENodeList nodes) throws ENodeCastException {		
		/*return ExpressionTools.sub(
				new ENode(nodes, new ENode(ENode.EMPTY)), 
				new ENode(filterWithStereotype(nodes, "id"), new ENode(ENode.EMPTY)) );*/
		return filterWithOutStereotype(nodes, "id");
	}

	/**
	 * Return only the primitive type. Test if the Meta Class is PrimitiveType or
	 * the name of the type of the attribute 
	 * @param c
	 * @return
	 * @throws ENodeCastException
	 */
	public List primitiveAttributes(Class c) throws ENodeCastException {		
		List result = new ArrayList();
		Iterator it = c.getAttributes().iterator();
		
		while (it.hasNext()){
			Property prop = (Property)it.next();
			String name = prop.getType().getName();
		
			if (	prop.getType() instanceof PrimitiveType ||
					name.equals("String") || 
					name.equals("Integer") || 
					name.equals("Date") || 
					name.equals("Boolean") ) {				
				result.add(prop);
			}
		}
		return result;
	}

	/**
	 * Find all subclass of a superclass. Browse all the model to find children.
	 * @param c The superclass
	 * @return A collection of org.eclipse.uml2.Class
	 * @throws ENodeCastException
	 * @throws FactoryException
	 */
	public Collection subClass(Class c) throws ENodeCastException, FactoryException {
		Element elt = c;
		do {
			elt = elt.getOwner();
		} while(elt.getOwner() != null || !(elt instanceof Model));
		
		if (!(elt instanceof Model)) {
			throw new ENodeCastException("Root node element is not a Model but is " + elt.eClass().getName());
		}
		
		// search all class of the model
		Collection subClasses = new ArrayList();
		for (Iterator iter = elt.eAllContents(); iter.hasNext();) {	
			EObject classObj = (EObject) iter.next();
			if (classObj instanceof Class) {
				Class subClass = (Class)classObj;
				// select only children
				if (subClass.getSuperClass(c.getName()) != null) {
					subClasses.add(subClass);
				}
			}
		}
		
		return subClasses;
	}

	/**
	 * Find all properties which are an association to this class. Browse all the model to find them.
	 * @param c The class pointed
	 * @return A collection of org.eclipse.uml2.Property
	 * @throws ENodeCastException
	 * @throws FactoryException
	 */
	public Collection pointedAttribute(Class c) throws ENodeCastException, FactoryException {
		Element elt = c;
		do {
			elt = elt.getOwner();
		} while(elt.getOwner() != null || !(elt instanceof Model));
		
		if (!(elt instanceof Model)) {
			throw new ENodeCastException("Root node element is not a Model but is " + elt.eClass().getName());
		}
		
		// search all class of the model
		Collection properties = new ArrayList();
		for (Iterator iter = elt.eAllContents(); iter.hasNext();) {
			EObject attObj = (EObject) iter.next();
			if (attObj instanceof Property) {
				Property prop = (Property)attObj;
				// select properties which have as target the c Class		
				if (prop.getType() == c) {
					properties.add(prop);
				}
			}
		}
		
		return properties;
	}
	
	/**
	 * Give to opposite attribute of an association.
	 * @param p
	 * @return
	 * @throws ENodeCastException
	 * @throws FactoryException
	 */
	public Property oppositeAttribute(Property p) throws ENodeCastException, FactoryException {
		Association asso = p.getAssociation();
		if (asso != null) {			
			EList list = asso.getMemberEnds();
			for (Iterator iter = list.iterator(); iter.hasNext();) {
				Property prop = (Property) iter.next();
				if (prop != p){
					return prop;
				}
			}
		}
		return null;
	}

	public Property oppositeAttributeOf(Association association, Class element) {
		EList list = association.getMemberEnds();
		for (Iterator iter = list.iterator(); iter.hasNext(); ) {
			Property prop = (Property) iter.next();
			if (!EcoreUtil.equals(prop.getType(),element))
				return prop;
		}
		return null;
	}

	/**
	 * Return a list of parameters of an operation without the return type
	 * @param node
	 * @return
	 * @throws ENodeCastException
	 * @throws FactoryException
	 */
	public ENodeList getRealParameters(ENode node) throws ENodeCastException, FactoryException {
		ENodeList parListFiltered = new ENodeList();
		if (node.getEObject() instanceof Operation){
			Operation op = (Operation)node.getEObject();
		
			EList parList = op.getOwnedParameters();
			for (Iterator iterPar = parList.iterator(); iterPar.hasNext();) {
				Parameter par = (Parameter) iterPar.next();
				if (!par.getDirection().equals(ParameterDirectionKind.RETURN_LITERAL))
					parListFiltered.add(new ENode(par, node));
			}
		}
		return parListFiltered;		
	}
	
	/**
	 * Return return parameter
	 * @param op
	 * @return
	 */
	public Parameter returnResult(Operation op) {
		return op.getReturnResult();
	}
	
	/**
	 * Return the list of class' associations
	 * @param element
	 * @return
	 */
	public Collection getAssociations(Class element) {
		return element.getAssociations();
}
	public boolean isNavigable(Property prop) {
		if (prop.getAssociation() != null)
			return prop.getAssociation().getNavigableOwnedEnds().contains(prop);
		else
			return true;
	}
}
