<%--
  Copyright � 2007 Obeo
  All rights reserved.  This program and the accompanying materials
  are made available under the terms of the Eclipse Public License 1.0
  
  You can apply any license to the files generated with this template
  and Acceleo.
  
  Original contributor : Pierre Alain Bourdil
--%>

<%
metamodel http://www.eclipse.org/uml2/2.0.0/UML
%>

<%script type="uml.Model" name="filePath" post="trim()"%>
<%name%>/dao/Dao.php5

<%script type="uml.Model" name="abstract_entity" file="<%filePath%>"%>
<?php
/**
 * @package dao
 */

/**
* PEAR log.
*/
require_once("Log.php");
	
/**
* PEAR DB
*/
require_once("DB.php");
/**
 * root class, implements execute query,checkError
 * @package dao
 * @version $Id$
 */
abstract class Dao {

	protected $logger;
	
	/** 
	* table name
	* @var string
	*/
	protected $tableName;
	
	
	/**
	* return $tableName;
	*/
	public function getTableName(){
		return $this->tableName;
	}
	
	/**
	* check for error and format message
	* @param object $dbObj a db object
	*/
	protected function checkError($dbObj, $query="") {
		if (PEAR::isError($dbObj)) {
			$logMsg = 	"\tMessage     : " . $dbObj->getMessage() . "\n" .
						"\tUser Info   : " . $dbObj->getUserInfo() . "\n".
						"\tDebug Info  : " . $dbObj->getDebugInfo() . "\n";
			$logMsg = "Error while executing query [" . $query . "] \n" . $logMsg;
			$this->logger->log($logMsg, PEAR_LOG_ERR);
			$this->logger->flush();
			// TODO throw an appropriate exception.
			throw new Exception();
		}
	}
	
	/**
	* Execute a query
	*
	* log at PEAR_LOG_DEBUG and check for error
	* @param object $connection a connection object
	* @param string $query the query
	* @return object $res the DB resultat object or false
	*/
	protected function executeQuery($connection, $query) {
		$this->logger->log("Executing query {" . $query . "}\n" . $this->getPrettyStackTrace(), PEAR_LOG_DEBUG);
		$this->logger->flush();
		$res =& $connection->getDb()->query($query);
		$this->checkError($res, $query);
		return $res;
	}
	
	/**
	 * Update the relations of child.
	 *
	 * @param object $connection the connection to the database.
	 * @param object $child the object to update.
	 * @param array $memory prevent prevents from multiple save of a same instance.
	 */
	public abstract function updateRelations($connection, $child, $memory = array());
	
	public abstract function create($connection, $entity, $memory = array());
	
	public abstract function update($connection, $entity, $memory = array());
	
	private function getPrettyStackTrace() {
		$dataInfo = debug_backtrace();
		$trace = "";
		for ($i=2; $i < 7 && $i < count($dataInfo) && $dataInfo[$i]["class"] != null; $i++) {
			$trace = $trace . "\t\tat " . $dataInfo[$i]["class"] . "->" . $dataInfo[$i]["function"] . "(line " . $dataInfo[$i-1]["line"] . ")\n";
		}
		return $trace;
	}
}


?>