package org.acceleo.module.pim.uml21.services;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeIterator;
import fr.obeo.acceleo.gen.template.eval.ENodeList;

public class EObjects {

	/**
	 * Return <code>true</code> if the stereotype <code>stereotype</code> is
	 * applied on the uml::Element <code>elt</code>.
	 * 
	 * @param elt
	 *            the uml::Element.
	 * @param stereotype
	 *            the name of the stereotype.
	 * @return <code>true</code> if the stereotype <code>stereotype</code>
	 *         is applied on the uml::Element <code>elt</code>.
	 * @throws ENodeCastException
	 */
	public boolean hasStereotype(Element elt, String stereotype)
			throws ENodeCastException {
		// search with real stereotype
		Stereotype stereotypeFound = null;
		Iterator iterStereotypes = elt.getAppliedStereotypes().iterator();
		while (iterStereotypes.hasNext() && stereotypeFound == null) {
			Stereotype currentStereotype = (Stereotype) iterStereotypes.next();
			if (currentStereotype.getName().equals(stereotype)) {
				stereotypeFound = currentStereotype;
			}
		}
		if (stereotypeFound == null) {
			// search with keywords
			if (elt.hasKeyword(stereotype))
				return true;

			return false;
		} else
			return true;
	}

	/**
	 * 
	 * @param package1
	 * @return
	 */
	public Collection getAllAssociations(Package package1) {
		Collection allAssoc = new LinkedList();
		Iterator iterMembers = package1.getOwnedMembers().iterator();
		while (iterMembers.hasNext()) {
			NamedElement currentMember = (NamedElement) iterMembers.next();
			if (currentMember instanceof Association) {
				allAssoc.add(currentMember);
			} else if (currentMember instanceof Package) {
				allAssoc.addAll(getAllAssociations((Package) currentMember));
			}
		}
		return allAssoc;
	}

	public Collection getProperties(org.eclipse.uml2.uml.Class class1) {
		Collection properties = new LinkedList();
		properties.addAll(class1.getAttributes());

		Iterator iterAssociations = this.getAllAssociations(class1.getModel())
				.iterator();

		while (iterAssociations.hasNext()) {
			Object object = iterAssociations.next();
			if (object instanceof Association) {
				Association association = (Association) object;
				if (association.getMemberEnds().size() == 2) {
					Property end1 = (Property) association.getMemberEnds().get(
							0);
					Property end2 = (Property) association.getMemberEnds().get(
							1);

					Property objetMetierEnd = null;
					Property inverseEnd = null;

					if (end1.getType() != null && end2.getType() != null) {

						if (end1.getType().equals(class1)
								&& end2.getType().equals(class1)) {
							// Association rflexive
							if (association.getNavigableOwnedEnds().contains(
									end1)) {
								properties.add(end1);
							}
							if (association.getNavigableOwnedEnds().contains(
									end2)) {
								properties.add(end2);
							}
						} else if (end1.getType().equals(class1)) {
							objetMetierEnd = end1;
							inverseEnd = end2;
						} else if (end2.getType().equals(class1)) {
							objetMetierEnd = end2;
							inverseEnd = end1;
						}

						if (objetMetierEnd != null
								&& association.getNavigableOwnedEnds()
										.contains(inverseEnd)) {
							properties.add(inverseEnd);
						}
					}
				}
			}
		}
		return properties;
	}

	public boolean isBidirectionnal(EObject object) throws ENodeCastException {
		// TODO isBidirectionnal  implmenter
		/*
		 * if (object instanceof StructuralFeature){ StructuralFeature feature =
		 * (StructuralFeature)object; return false; } else { return false; }
		 */
		return false;
	}

	public boolean isNtoNAssociation(EObject object) throws ENodeCastException {
		return false;
	}

	/**
	 * Filter a list of Element which have to stereotype specify as parameter
	 * 
	 * @param nodes
	 * @param stereotype
	 * @throws ENodeCastException
	 */
	public ENodeList filterWithStereotype(ENodeList nodes, String stereotype)
			throws ENodeCastException {
		ENodeList result = new ENodeList();
		ENodeIterator it = nodes.iterator();
		while (it.hasNext()) {
			ENode node = it.next();
			if (node.isEObject() && node.getEObject() instanceof Element) {
				Element elt = (Element) node.getEObject();
				if (hasStereotype(elt, stereotype)) {
					result.add(node);
					System.out.println("With : "
							+ ((NamedElement) elt).getName());
				}
			}
		}
		return result;
	}

	// TODO :  enlever aprs correction bug 58 et  laisser dans .mt
	public ENodeList attributeIds(ENodeList nodes) throws ENodeCastException {
		return filterWithStereotype(nodes, "id");
	}

	public ENodeList filterWithOutStereotype(ENodeList nodes, String stereotype)
			throws ENodeCastException {
		ENodeList result = new ENodeList();
		ENodeIterator it = nodes.iterator();
		while (it.hasNext()) {
			ENode node = it.next();
			if (node.isEObject() && node.getEObject() instanceof Element) {
				Element elt = (Element) node.getEObject();
				if (!hasStereotype(elt, stereotype)) {
					result.add(node);
					System.out.println("Without : "
							+ ((NamedElement) elt).getName());
				}
			}
		}
		return result;
	}

	// TODO :  enlever aprs correction bug 58
	public ENodeList attributeNotIds(ENodeList nodes) throws ENodeCastException {
		/*
		 * return ExpressionTools.sub( new ENode(nodes, new ENode(ENode.EMPTY)),
		 * new ENode(filterWithStereotype(nodes, "id"), new ENode(ENode.EMPTY)) );
		 */
		return filterWithOutStereotype(nodes, "id");
	}

	/**
	 * Return return parameter
	 * 
	 * @param op
	 * @return
	 */
	public Parameter returnResult(Operation op) {
		return op.getReturnResult();
	}

	public String stereotype(Element elt) {
		EList stereos = elt.getAppliedStereotypes();
		if (stereos == null || stereos.size() == 0) {
			EList kws = elt.getKeywords();
			if (kws == null || kws.size() == 0) {
				return "";
			} else {
				return kws.get(0).toString();
			}
		} else {
			return ((Stereotype) stereos.get(0)).getName();
		}
	}

	/**
	 * Return the inverse end of the property <code>property</code>.
	 * 
	 * @param property
	 *            the property.
	 * @return the inverse end of the property <code>property</code>.
	 */
	public Property inverse(Property property) {
		Property inverse = null;
		if (property.getAssociation() != null) {
			if (property.getAssociation().getMemberEnds().size() == 2) {
				Property firstEnd = (Property) property.getAssociation()
						.getMemberEnds().get(0);
				Property secondEnd = (Property) property.getAssociation()
						.getMemberEnds().get(1);
				if (property.equals(firstEnd)) {
					inverse = secondEnd;
				} else if (property.equals(secondEnd)) {
					inverse = firstEnd;
				}
			}
		}
		return inverse;
	}
}
