package org.acceleo.module.pim.uml21.services;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Property;

public class SqlTypeServices {

	/**
	 * Convert a standard name to SQL convention.<br/> Exemple : myAttribute ->
	 * MY_ATTRIBUTE
	 * 
	 * @param umlName
	 * @return
	 */
	public String toSqlNotation(String umlName) {
		Pattern p = Pattern.compile("(\\p{Lower})(\\p{Upper})");
		Matcher m = p.matcher(umlName);
		String sqlName = m.replaceAll("$1_$2");
		sqlName = sqlName.toUpperCase();
		return sqlName;
	}

	public boolean isBiAssociationSource(Property prop) {
		Class contClass = (Class) prop.eContainer();
		Class targetClass = (Class) prop.getType();
		if (contClass.getName().compareTo(targetClass.getName()) > 0)
			return true;
		else
			return false;
	}

	private static Map umlType2SqlTypes = new HashMap();
	static {
		umlType2SqlTypes.put("string", "VARCHAR(200)");
		umlType2SqlTypes.put("integer", "INTEGER");
		umlType2SqlTypes.put("int", "INTEGER");
		umlType2SqlTypes.put("date", "DATE");
		umlType2SqlTypes.put("text", "TEXT");
	}

	public String getSqlType(PrimitiveType type) {
		return (String) SqlTypeServices.umlType2SqlTypes.get(type.getName()
				.toLowerCase());
	}

	/**
	 * 
	 * @return
	 */
	public Collection getTableRelations(Class class1) {
		Collection tableRelations = new LinkedList();
		Iterator iterAssociations = class1.getAssociations().iterator();
		while (iterAssociations.hasNext()) {
			Association currentAssociation = (Association) iterAssociations
					.next();

			if (currentAssociation.getMemberEnds().size() == 2) {
				Property firstEnd = (Property) currentAssociation
						.getMemberEnds().get(0);
				Property secondEnd = (Property) currentAssociation
						.getMemberEnds().get(1);

				Property class1End = null;
				Property otherEnd = null;
				if (firstEnd.getType().equals(class1)) {
					class1End = firstEnd;
					otherEnd = secondEnd;
				} else if (secondEnd.getType().equals(class1)) {
					class1End = secondEnd;
					otherEnd = firstEnd;
				}

				if (class1End != null) {
					if (class1End.getUpper() != 1 && otherEnd.getUpper() == 1) {
						// (class1End) * --- ?..1 (otherEnd)
						tableRelations.add(class1End);
					} else if (class1End.getUpper() == 1
							&& class1End.getLower() == 0
							&& otherEnd.getLower() == 1
							&& otherEnd.getUpper() == 1) {
						// (class1End) 0..1 --- 1 (otherEnd)
						tableRelations.add(class1End);
					} else if (class1End.getUpper() == 1
							&& otherEnd.getUpper() == 1
							&& class1End.getLower() == otherEnd.getLower()) {
						if (class1End == firstEnd) {
							tableRelations.add(class1End);
						}
					}

				}

			}

		}

		return tableRelations;
	}

	public Collection getAssociationsTables(Model model) {
		Collection associationsTables = new LinkedList();
		EObjects eObjects = new EObjects();
		Iterator iterAssociations = eObjects.getAllAssociations(model)
				.iterator();
		while (iterAssociations.hasNext()) {
			Association currentAssociation = (Association) iterAssociations
					.next();
			if (currentAssociation.getMembers().size() == 2) {
				Property firstEnd = (Property) currentAssociation
						.getMemberEnds().get(0);
				Property secondEnd = (Property) currentAssociation
						.getMemberEnds().get(1);
				if (firstEnd.getUpper() != 1 && secondEnd.getUpper() != 1) {
					associationsTables.add(currentAssociation);
				}
			}
		}
		return associationsTables;
	}

}
