<?php
require_once 'PHPUnit/Framework.php';
require_once('modules-available/testmodule/Dto/A.php') ;
require_once('modules-available/testmodule/Dto/B.php') ;
require_once('modules-available/testmodule/Dto/C.php') ;
require_once('modules-available/testmodule/Dto/D.php') ;
require_once('modules-available/testmodule/Dto/E.php') ;
require_once('lib/Wisss/Dto/Registry/Abstract.php') ;

class DaoTest extends PHPUnit_Framework_TestCase
{
	public function setUp()
	{
		$db = Zend_Registry::get('db') ;
   		
		$queries = explode(';',file_get_contents('install/mysql_uninstall.sql')) ;
		foreach($queries as $query) {
		$escaped_query = trim($query) ;
			if(!empty($escaped_query)) {
				$db->query($escaped_query) ;
			}
		}
		
		$queries = explode(';',file_get_contents('install/mysql_install.sql')) ;
		foreach($queries as $query) {
		$escaped_query = trim($query) ;
			if(!empty($escaped_query)) {
				$db->query($escaped_query) ;
			}
		}		
	}
	
	public function testObjectInsert()
	{
		$instance_of_a = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_of_a->setAtta('test value') ;
		$instance_of_a->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_of_a);
		
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_of_a->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_of_a,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_of_a,get_class($instance_of_a))) ;
		
		$new_instance->__delete() ;
	}
	
	public function testObjectInsertWithInheritance()
	{
		$instance_of_d = Wisss_Dto_Registry_Abstract::getDto('testmodule','D') ;
		$instance_of_d->setAtta('test value') ;
		$instance_of_d->setAttrd('2007-12-25 00:00:00') ;
		$instance_of_d->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_of_d);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','D',$instance_of_d->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_of_d,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_of_d,get_class($instance_of_d))) ;
		
		$new_instance->__delete() ;
	}
	
	public function testObjectInsertWithArrayAttribute()
	{
		$instance_of_c = Wisss_Dto_Registry_Abstract::getDto('testmodule','C') ;
		$instance_of_c->setListOfAttc(array(true,false,true)) ;
		$instance_of_c->addAttc(false) ;
		$instance_of_c->addAttc(true) ;
		$instance_of_c->removeAttc(false) ;
		$instance_of_c->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_of_c);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','C',$instance_of_c->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_of_c,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_of_c,get_class($instance_of_c))) ;
		
		$new_instance->__delete() ;
	}
	
	public function testObjectInsertWithInheritedArrayAttribute()
	{
		$instance_of_e = Wisss_Dto_Registry_Abstract::getDto('testmodule','E') ;
		$instance_of_e->setListOfAttc(array(true,false,true)) ;
		$instance_of_e->addAttc(false) ;
		$instance_of_e->addAttc(true) ;
		$instance_of_e->setAtte(3) ;
		$instance_of_e->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_of_e);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','E',$instance_of_e->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_of_e,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_of_e,get_class($instance_of_e))) ;
		
		$new_instance->__delete() ;
	}
	
	public function testObjectUpdate()
	{
		$instance_of_a = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_of_a->setAtta('test value') ;
		$instance_of_a->__save() ;
		$instance_of_a->setAtta('value test') ;
		$instance_of_a->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_of_a);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_of_a->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_of_a,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_of_a,get_class($instance_of_a))) ;
		
		$new_instance->__delete() ;
	}
	
	public function testObjectUpdateWithInheritance()
	{
		$instance_of_d = Wisss_Dto_Registry_Abstract::getDto('testmodule','D') ;
		$instance_of_d->setAtta('test value') ;
		$instance_of_d->setAttrd('2007-12-24 00:00:00') ;
		$instance_of_d->__save() ;
		$instance_of_d->setAtta('value test') ;
		$instance_of_d->setAttrd('2007-12-25 00:00:00') ;
		$instance_of_d->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_of_d);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','D',$instance_of_d->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_of_d,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_of_d,get_class($instance_of_d))) ;
		
		$new_instance->__delete() ;
	}
	
	public function testObjectUpdateWithArrayAttribute()
	{
		$instance_of_c = Wisss_Dto_Registry_Abstract::getDto('testmodule','C') ;
		$instance_of_c->setListOfAttc(array(true,false,true)) ;
		$instance_of_c->addAttc(false) ;
		$instance_of_c->addAttc(true) ;
		$instance_of_c->removeAttc(false) ;
		$instance_of_c->__save() ;
		$instance_of_c->setListOfAttc(array(true,true)) ;
		$instance_of_c->addAttc(false) ;
		$instance_of_c->addAttc(false) ;
		$instance_of_c->removeAttc(false) ;
		$instance_of_c->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_of_c);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','C',$instance_of_c->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_of_c,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_of_c,get_class($instance_of_c))) ;
		
		$new_instance->__delete() ;
	}
	
	public function testObjectUpdateWithInheritedArrayAttribute()
	{
		$instance_of_e = Wisss_Dto_Registry_Abstract::getDto('testmodule','E') ;
		$instance_of_e->setListOfAttc(array(true,false,true)) ;
		$instance_of_e->addAttc(false) ;
		$instance_of_e->addAttc(true) ;
		$instance_of_e->setAtte(3) ;
		$instance_of_e->__save() ;
		$instance_of_e->setListOfAttc(array(true,true)) ;
		$instance_of_e->addAttc(false) ;
		$instance_of_e->addAttc(false) ;
		$instance_of_e->removeAttc(true) ;
		$instance_of_e->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_of_e);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','E',$instance_of_e->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_of_e,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_of_e,get_class($instance_of_e))) ;
		
		$new_instance->__delete() ;
	}
	
	public function testObjectRef1_1Insert()
	{
		$instance_a = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_a->setAtta('test value') ;
		$instance_b = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b->setAttb(5) ;
		$instance_a->setRolebInAsso11($instance_b) ;
		$instance_a->__save() ;
		$instance_b->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_a);
		Wisss_Dto_Registry_Abstract::unregister($instance_b);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->getRolebInAsso11()->__setToLoaded() ;
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_a,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_a,get_class($instance_a))) ;
		$this->assertTrue($change_recorder->isSync($instance_b,get_class($instance_b))) ;
		
		$new_instance->getRolebInAsso11()->__delete() ;
		$instance_a->deleteObservers() ;
		$instance_b->deleteObservers() ;
		$new_instance->__delete() ;
		$new_instance->deleteObservers() ;
	}
	
	public function testObjectRef1_1AttributesUpdate()
	{
		$instance_a = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_a->setAtta('test value') ;
		$instance_b = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b->setAttb(12) ;
		$instance_a->setRolebInAsso11($instance_b) ;
		$instance_a->__save() ;
		$instance_b->__save() ;
		$instance_a->setAtta('value test') ;
		$instance_b->setAttb(6) ;
		$instance_a->__save() ;
		$instance_b->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_a);
		Wisss_Dto_Registry_Abstract::unregister($instance_b);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->getRolebInAsso11()->__setToLoaded() ;
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_a,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_a,get_class($instance_a))) ;
		$this->assertTrue($change_recorder->isSync($instance_b,get_class($instance_b))) ;
		
		$instance_a->deleteObservers() ;
		$instance_b->deleteObservers() ;
		$new_instance->__delete() ;
		$new_instance->getRolebInAsso11()->__delete() ;
		$new_instance->deleteObservers() ;
	}
	
	public function testObjectRef1_1ReplaceUpdate()
	{
		$instance_a = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_a->setAtta('test value') ;
		$instance_b1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b1->setAttb(5) ;
		$instance_a->setRolebInAsso11($instance_b1) ;
		$instance_a->__save() ;
		$instance_b1->__save() ;
		
		$instance_a->setAtta('value test') ;
		$instance_b2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b2->setAttb(3) ;
		$instance_a->setRolebInAsso11($instance_b2) ;
		$instance_a->__save() ;
		$instance_b2->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_a);
		Wisss_Dto_Registry_Abstract::unregister($instance_b1);
		Wisss_Dto_Registry_Abstract::unregister($instance_b2);
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a->getId()) ;
		$new_instance->__load() ;
		
		$new_instance->getRolebInAsso11()->__setToLoaded() ;
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_a,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_a,get_class($instance_a))) ;
		$this->assertTrue($change_recorder->isSync($instance_b1,get_class($instance_b1))) ;
		$this->assertTrue($change_recorder->isSync($instance_b2,get_class($instance_b2))) ;
		
		$instance_a->deleteObservers() ;
		$instance_b1->__delete() ;
		$instance_b1->deleteObservers() ;
		$instance_b2->deleteObservers() ;
		$new_instance->__delete() ;
		$new_instance->getRolebInAsso11()->__delete() ;
		$new_instance->deleteObservers() ;
	}
	
	public function testObjectRef1_nInsert()
	{
		$instance_a = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_a->setAtta('test value') ;
		$instance_b1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b1->setAttb(5) ;
		$instance_b2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b2->setAttb(3) ;
		$instance_a->setListOfRolebInAsso1N(array($instance_b1,$instance_b2)) ;
		$instance_a->__save() ;
		$instance_b1->__save() ;
		$instance_b2->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_a);
		Wisss_Dto_Registry_Abstract::unregister($instance_b1);
		Wisss_Dto_Registry_Abstract::unregister($instance_b2);
		
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a->getId()) ;
		$new_instance->__load() ;
		
		foreach($new_instance->getListOfRolebInAsso1N() as $b) {
			$b->__setToLoaded() ;			
		}
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_a,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_a,get_class($instance_a))) ;
		$this->assertTrue($change_recorder->isSync($instance_b1,get_class($instance_b1))) ;
		$this->assertTrue($change_recorder->isSync($instance_b2,get_class($instance_b2))) ;
		
		$instance_a->deleteObservers() ;
		$instance_b1->deleteObservers() ;
		$instance_b2->deleteObservers() ;
		$new_instance->deleteObservers() ;
		
		foreach($new_instance->getListOfRolebInAsso1N() as $b) {
			$b->__delete() ;
			$b->deleteObservers() ;
		}
		$new_instance->__delete() ;
	}
	
	public function testObjectRef1_nUpdate()
	{
		$instance_a = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_a->setAtta('test value') ;
		$instance_b1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b1->setAttb(5) ;
		$instance_b2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b2->setAttb(3) ;
		$instance_a->setListOfRolebInAsso1N(array($instance_b1,$instance_b2)) ;
		$instance_a->__save() ;
		$instance_b1->__save() ;
		$instance_b2->__save() ;
		$instance_a->removeRolebInAsso1N($instance_b1);
		$instance_b3 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b3->setAttb(6) ;
		$instance_a->addRolebInAsso1N($instance_b3);
		$instance_a->__save() ;
		$instance_b1->__save() ;
		$instance_b2->__save() ;
		$instance_b3->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_a);
		Wisss_Dto_Registry_Abstract::unregister($instance_b1);
		Wisss_Dto_Registry_Abstract::unregister($instance_b2);
		Wisss_Dto_Registry_Abstract::unregister($instance_b3);
		
		$new_instance = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a->getId()) ;
		$new_instance->__load() ;
		
		foreach($new_instance->getListOfRolebInAsso1N() as $b) {
			$b->__setToLoaded() ;			
		}
		$new_instance->__setToLoaded();
		$this->assertEquals($instance_a,$new_instance) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_a,get_class($instance_a))) ;
		$this->assertTrue($change_recorder->isSync($instance_b1,get_class($instance_b1))) ;
		$this->assertTrue($change_recorder->isSync($instance_b2,get_class($instance_b2))) ;
		$this->assertTrue($change_recorder->isSync($instance_b2,get_class($instance_b3))) ;
		
		$instance_a->deleteObservers() ;
		$instance_b1->deleteObservers() ;
		$instance_b2->deleteObservers() ;
		$new_instance->deleteObservers() ;
		
		foreach($new_instance->getListOfRolebInAsso1N() as $b) {
			$b->__delete() ;
			$b->deleteObservers() ;
		}
		$instance_b1->__delete() ;
		$new_instance->__delete() ;
	}
	
	public function testObjectRefn_nInsert()
	{
		$instance_a1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_a1->setAtta('test value3') ;
		$instance_a2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_a2->setAtta('test value4') ;
		$instance_b1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b1->setAttb(12) ;
		$instance_b2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b2->setAttb(10) ;
		$instance_a1->setListOfRolebInAssoNN(array($instance_b1,$instance_b2)) ;
		$instance_a2->setListOfRolebInAssoNN(array($instance_b1,$instance_b2)) ;
		$instance_a1->__save() ;
		$instance_a2->__save() ;
		$instance_b1->__save() ;
		$instance_b2->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_a1) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_a2) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b1) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b2) ;
		$new_instance1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a1->getId()) ;
		$new_instance1->__load() ;
		$new_instance2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a2->getId()) ;
		$new_instance2->__load() ;
		
		foreach($new_instance1->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		foreach($new_instance2->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		
		$new_instance1->__setToLoaded();
		$new_instance2->__setToLoaded();
		$this->assertEquals($instance_a1,$new_instance1) ;
		$this->assertEquals($instance_a2,$new_instance2) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_a1,get_class($instance_a1))) ;
		$this->assertTrue($change_recorder->isSync($instance_a2,get_class($instance_a2))) ;
		
		$instance_a1->deleteObservers() ;
		$instance_a2->deleteObservers() ;
		$instance_b1->deleteObservers() ;
		$instance_b2->deleteObservers() ;
		$new_instance1->deleteObservers() ;
		$new_instance2->deleteObservers() ;
		foreach($new_instance1->getListOfRolebInAssoNN() as $b) {
			$b->__delete() ;
			$b->deleteObservers() ;
		}
		$new_instance1->__delete() ;
		$new_instance2->__delete() ;
	}
	
	public function testObjectRefn_nUpdate()
	{
		$instance_a1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_a1->setAtta('test value3') ;
		$instance_a2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		$instance_a2->setAtta('test value4') ;
		$instance_b1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b1->setAttb(12) ;
		$instance_b2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b2->setAttb(10) ;
		$instance_a1->setListOfRolebInAssoNN(array($instance_b1,$instance_b2)) ;
		$instance_a2->setListOfRolebInAssoNN(array($instance_b1,$instance_b2)) ;
		$instance_a1->__save() ;
		$instance_a2->__save() ;
		$instance_b1->__save() ;
		$instance_b2->__save() ;
		
		$instance_b2->removeRoleaInAssoNN($instance_a1) ;
		$instance_b3 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		$instance_b3->setAttb(15) ;
		$instance_a2->addRolebInAssoNN($instance_b3) ;
		$instance_a1->__save() ;
		$instance_b2->__save() ;
		$instance_b3->__save() ;
		
		Wisss_Dto_Registry_Abstract::unregister($instance_a1) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_a2) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b1) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b2) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b3) ;
		$new_instance1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a1->getId()) ;
		$new_instance1->__load() ;
		$new_instance2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a2->getId()) ;
		$new_instance2->__load() ;
		
		foreach($new_instance1->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		foreach($new_instance2->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		
		$new_instance1->__setToLoaded();
		$new_instance2->__setToLoaded();
		
		$this->assertEquals($instance_a1,$new_instance1) ;
		$this->assertEquals($instance_a2,$new_instance2) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_a1,get_class($instance_a1))) ;
		$this->assertTrue($change_recorder->isSync($instance_a2,get_class($instance_a2))) ;
		$this->assertTrue($change_recorder->isSync($instance_b1,get_class($instance_b1))) ;
		$this->assertTrue($change_recorder->isSync($instance_b2,get_class($instance_b2))) ;
		$this->assertTrue($change_recorder->isSync($instance_b3,get_class($instance_b3))) ;
		
		$instance_a1->deleteObservers() ;
		$instance_a2->deleteObservers() ;
		$instance_b1->deleteObservers() ;
		$instance_b2->deleteObservers() ;
		$instance_b3->deleteObservers() ;
		$instance_a1->__delete() ;
		$instance_a2->__delete() ;
		$instance_b1->__delete() ;
		$instance_b2->__delete() ;
		$instance_b3->__delete() ;
	}
	
	public function testSync()
	{
		$instance_a1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		Wisss_Dto_Registry_Abstract::register($instance_a1) ;
		$instance_a1->setAtta('test value12') ;
		$instance_a2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		Wisss_Dto_Registry_Abstract::register($instance_a2) ;
		$instance_a2->setAtta('test value45') ;
		$instance_b1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b1) ;
		$instance_b1->setAttb(34) ;
		$instance_b2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b2) ;
		$instance_b2->setAttb(12) ;
		$instance_a1->setListOfRolebInAssoNN(array($instance_b1,$instance_b2)) ;
		$instance_a2->setListOfRolebInAssoNN(array($instance_b1,$instance_b2)) ;
		
		$instance_b2->removeRoleaInAssoNN($instance_a1) ;
		$instance_b3 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b3) ;
		$instance_b3->setAttb(15) ;
		$instance_a2->addRolebInAssoNN($instance_b3) ;
		
		$instance_b4 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b4) ;
		$instance_b4->setAttb(5) ;
		$instance_b5 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b5) ;
		$instance_b5->setAttb(3) ;
		$instance_a1->setListOfRolebInAsso1N(array($instance_b4,$instance_b5)) ;
		
		Wisss_Dto_Registry_Abstract::sync() ;

		Wisss_Dto_Registry_Abstract::unregister($instance_a1) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_a2) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b1) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b2) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b3) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b4) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b5) ;
		$new_instance1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a1->getId()) ;
		$new_instance1->__load() ;
		$new_instance2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a2->getId()) ;
		$new_instance2->__load() ;
		
		foreach($new_instance1->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		foreach($new_instance1->getListOfRolebInAsso1N() as $b) {
			$b->__setToLoaded() ;
		}
		foreach($new_instance2->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		
		$new_instance1->__setToLoaded();
		$new_instance2->__setToLoaded();
		$this->assertEquals($instance_a1,$new_instance1) ;
		$this->assertEquals($instance_a2,$new_instance2) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_a1,get_class($instance_a1))) ;
		$this->assertTrue($change_recorder->isSync($instance_a2,get_class($instance_a2))) ;
		$this->assertTrue($change_recorder->isSync($instance_b1,get_class($instance_b1))) ;
		$this->assertTrue($change_recorder->isSync($instance_b2,get_class($instance_b2))) ;
		$this->assertTrue($change_recorder->isSync($instance_b3,get_class($instance_b3))) ;
		$this->assertTrue($change_recorder->isSync($instance_b3,get_class($instance_b4))) ;
		$this->assertTrue($change_recorder->isSync($instance_b3,get_class($instance_b5))) ;
		
		$instance_a1->deleteObservers() ;
		$instance_a2->deleteObservers() ;
		$instance_b1->deleteObservers() ;
		$instance_b2->deleteObservers() ;
		$instance_b3->deleteObservers() ;
		$instance_b4->deleteObservers() ;
		$instance_b5->deleteObservers() ;
		$instance_a1->__delete() ;
		$instance_a2->__delete() ;
		$instance_b1->__delete() ;
		$instance_b2->__delete() ;
		$instance_b3->__delete() ;
		$instance_b4->__delete() ;
		$instance_b5->__delete() ;
	}
	
	public function testSyncAfterReload()
	{
		$instance_a1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		Wisss_Dto_Registry_Abstract::register($instance_a1) ;
		$instance_a1->setAtta('test value12') ;
		$instance_a2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		Wisss_Dto_Registry_Abstract::register($instance_a2) ;
		$instance_a2->setAtta('test value45') ;
		$instance_b1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b1) ;
		$instance_b1->setAttb(34) ;
		$instance_b2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b2) ;
		$instance_b2->setAttb(12) ;
		$instance_a1->setListOfRolebInAssoNN(array($instance_b1,$instance_b2)) ;
		$instance_a2->setListOfRolebInAssoNN(array($instance_b1,$instance_b2)) ;
		
		$instance_b2->removeRoleaInAssoNN($instance_a1) ;
		$instance_b3 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b3) ;
		$instance_b3->setAttb(15) ;
		$instance_a2->addRolebInAssoNN($instance_b3) ;
		
		$instance_b4 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b4) ;
		$instance_b4->setAttb(5) ;
		$instance_b5 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b5) ;
		$instance_b5->setAttb(3) ;
		$instance_a1->setListOfRolebInAsso1N(array($instance_b4,$instance_b5)) ;
		
		Wisss_Dto_Registry_Abstract::sync() ;

		Wisss_Dto_Registry_Abstract::unregister($instance_a1) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_a2) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b1) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b2) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b3) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b4) ;
		Wisss_Dto_Registry_Abstract::unregister($instance_b5) ;
		$new_instance1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a1->getId()) ;
		$new_instance1->__load() ;
		$new_instance2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a2->getId()) ;
		$new_instance2->__load() ;
		
		$new_instance1->getListOfRolebInAssoNN() ;
		$new_instance1->getListOfRolebInAsso1N() ;
		$new_instance2->getListOfRolebInAssoNN() ;
		
		$instance_b6 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b6) ;
		$instance_b6->setAttb(9) ;
		$instance_b7 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b7) ;
		$instance_b7->setAttb(11) ;
		$new_instance1->AddRolebInAssoNN($instance_b6) ;
		$new_instance1->AddRolebInAssoNN($instance_b7) ;
		
		Wisss_Dto_Registry_Abstract::sync() ;
		
		Wisss_Dto_Registry_Abstract::clear() ;
		
		$new_new_instance1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a1->getId()) ;
		$new_new_instance1->__load() ;
		$new_new_instance2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a2->getId()) ;
		$new_new_instance2->__load() ;
		
		foreach($new_instance1->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		foreach($new_instance1->getListOfRolebInAsso1N() as $b) {
			$b->__setToLoaded() ;
		}
		foreach($new_instance2->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		
		$new_instance1->__setToLoaded();
		$new_instance2->__setToLoaded();
		
		foreach($new_new_instance1->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		foreach($new_new_instance1->getListOfRolebInAsso1N() as $b) {
			$b->__setToLoaded() ;
		}
		foreach($new_new_instance2->getListOfRolebInAssoNN() as $b) {
			$b->__setToLoaded() ;
		}
		
		$new_new_instance1->__setToLoaded();
		$new_new_instance2->__setToLoaded();
		
		$this->assertEquals($new_instance1,$new_new_instance1) ;
		$this->assertEquals($new_instance2,$new_new_instance2) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		$this->assertTrue($change_recorder->isSync($instance_a1,get_class($instance_a1))) ;
		$this->assertTrue($change_recorder->isSync($instance_a2,get_class($instance_a2))) ;
		$this->assertTrue($change_recorder->isSync($instance_b1,get_class($instance_b1))) ;
		$this->assertTrue($change_recorder->isSync($instance_b2,get_class($instance_b2))) ;
		$this->assertTrue($change_recorder->isSync($instance_b3,get_class($instance_b3))) ;
		$this->assertTrue($change_recorder->isSync($instance_b3,get_class($instance_b4))) ;
		$this->assertTrue($change_recorder->isSync($instance_b3,get_class($instance_b5))) ;
		
		$instance_a1->deleteObservers() ;
		$instance_a2->deleteObservers() ;
		$instance_b1->deleteObservers() ;
		$instance_b2->deleteObservers() ;
		$instance_b3->deleteObservers() ;
		$instance_b4->deleteObservers() ;
		$instance_b5->deleteObservers() ;
		$instance_b6->deleteObservers() ;
		$instance_b7->deleteObservers() ;
		$instance_a1->__delete() ;
		$instance_a2->__delete() ;
		$instance_b1->__delete() ;
		$instance_b2->__delete() ;
		$instance_b3->__delete() ;
		$instance_b4->__delete() ;
		$instance_b5->__delete() ;
		$instance_b6->__delete() ;
		$instance_b7->__delete() ;
	}
	
	public function testLoadFromRegistry()
	{
		$instance_a1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		Wisss_Dto_Registry_Abstract::register($instance_a1) ;
		$instance_a1->setAtta('a1') ;
		$instance_a2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A') ;
		Wisss_Dto_Registry_Abstract::register($instance_a2) ;
		$instance_a2->setAtta('a2') ;
		$instance_b1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','B') ;
		Wisss_Dto_Registry_Abstract::register($instance_b1) ;
		$instance_b1->setAttb(1) ;
		$instance_a1->setListOfRolebInAssoNN(array($instance_b1)) ;
		$instance_a2->setListOfRolebInAssoNN(array($instance_b1)) ;
		
		Wisss_Dto_Registry_Abstract::sync() ;
		Wisss_Dto_Registry_Abstract::clear() ;
		
		$new_instance1 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a1->getId()) ;
		$new_instance1->__load() ;
		$new_instance2 = Wisss_Dto_Registry_Abstract::getDto('testmodule','A',$instance_a2->getId()) ;
		$new_instance2->__load() ;
		
		$b_from_instance1 = $new_instance1->getListOfRolebInAssoNN() ;
		$b_from_instance2 = $new_instance2->getListOfRolebInAssoNN() ;
		
		$b1 = $b_from_instance1[0] ;
		$b2 = $b_from_instance2[0] ;
		
		$this->assertTrue($b1 === $b2) ;
		$change_recorder = Zend_Registry::get('changeRecorder') ;
		
		$instance_a1->deleteObservers() ;
		$instance_a2->deleteObservers() ;
		$instance_b1->deleteObservers() ;
		$new_instance1->deleteObservers() ;
		$new_instance2->deleteObservers() ;
		$b1->__delete() ;
		$new_instance1->__delete() ;
		$new_instance2->__delete() ;
	}
}
?>