<%
metamodel http://www.acceleo.org/module/wisss/webapp
%>

<%script type="webapp.Webapp" name="abstract_dto_registry" file="lib/Wisss/Dto/Registry/Abstract.php"%>
<?php
require_once ('Zend/Registry.php') ;

class Wisss_Dto_Registry_Exception extends Exception
{
	public function __construct($message, $code = 0) {
        parent::__construct($message, $code);
    }

    public function __toString() {
        return __CLASS__ . ": [{$this->code}]: {$this->message}\n";
    }
}

/**
 * class Wisss_Dto_Registry_Abstract
 * @package Wisss
 */
abstract class Wisss_Dto_Registry_Abstract
{
    private static $registry = array() ;
    private static $pending = array() ;
    
    public static function register($object)
    {
    	$object_id = $object->getId() ;
    	if(empty($object_id)) {
    		self::$pending[$object->__getUid()] = $object ;
    	} else {
    		if(array_key_exists($object->__getUid(),self::$pending)) {
    			unset(self::$pending[$object->__getUid()]) ;
    		}
	    	$old = self::find($object->getId()) ;
	    	if(!isset($old)) {
	    		self::$registry[$object->getId()] = $object ;
			} else if($old !== $object) {
				throw new Wisss_Dto_Registry_Exception('Trying to overwrite a registered DTO : '.$old->__getUid().' by '.$object->__getUid(),1) ;
			}
		}
		// if the object is already registered, we ignore
    }
    
    public static function unregister($object)
    {
    	if(array_key_exists($object->getId(),self::$registry)) {
    		unset(self::$registry[$object->getId()]) ;
		}
    }
    
    public static function clear()
    {
    	self::$registry = array() ;
    	self::$pending = array() ;
    }
    
    public static function find($id)
    {
    	if(array_key_exists($id,self::$registry)) {
    		return self::$registry[$id] ;
    	} else {
    		return null ;
    	}
    }
    
    /**
	 * synchronize all modified objects to their storage
	 */
	 public static function sync()
	 {
	 	$change_recorder = Zend_Registry::get('changeRecorder') ;
	 	foreach(self::$registry as $object) {
	 		if(!$change_recorder->isSync($object)) {
	 			$object->__save() ;
	 		}
	 	}
	 	
	 	foreach(self::$pending as $object) {
	 		if(!$change_recorder->isSync($object)) {
	 			$object->__save() ;
	 		}
	 	}
	 }
	 
	 /**
     * returns a dto for the specified class and module
     *
     * @param string $module
     * @param string $class
     * @return Wisss_Dto_Abstract
     */
    public static function getDto($module,$class,$id=null)
    {
    	$new_dto = null ;
    	$module = strtolower($module) ;
    	$class = self::toCamelCase($class) ;
        $dto_class_name = $module.'_Dto_'.$class ;
        
        /** 
         *  if an id is provided, we ask the dto registry 
         *  if the dto already exists in memory 
         */
    	if(isset($id)) {
    		$new_dto = Wisss_Dto_Registry_Abstract::find($id) ;
    	}
    	if(!isset($new_dto)) {
	        Zend_Loader::loadClass($dto_class_name,array('modules-available'));
	        $new_dto = new $dto_class_name() ;
	        $new_dto->addObserver(Zend_Registry::get('changeRecorder')) ;
        }
        if(isset($id)) {
        	$new_dto->setId($id) ;
        }
        return $new_dto ;
    }
    
    private static function toCamelCase($str)
    {
		$words = explode(" ",$str) ;
		$camelCasedStr = "" ;
		foreach($words as $word) {
			$camelCasedStr .= strtoupper(substr($word,0,1)).strtolower(substr($word,1)) ;
		}
		$words2 = explode("_",$str) ;
		$camelCasedStr2 = "" ;
		foreach($words2 as $word2) {
			$camelCasedStr2 .= strtoupper(substr($word2,0,1)).strtolower(substr($word2,1)) ;
		}
		return $camelCasedStr2 ;
    }
}
