/*
  Copyright © 2007 Makina Corpus
  All rights reserved.  This program and the accompanying materials
  are made available under the terms of the GNU Public License v2.0
  which accompanies this distribution, and is available at
  http://www.gnu.org/licenses/gpl.txt
  You can apply any license to the files generated with this template
  and Acceleo.

@author  Francois Gaudin <francois.gaudin@makina-corpus.com> 
*/

package services;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import wisss.data.Association;
import wisss.data.AssociationEnd;
import wisss.data.Attribute;
import wisss.data.BinaryAssociation;
import wisss.data.Class;
import wisss.data.Module;
import wisss.data.ModuleElement;

public class NamingServices {
		private String toSqlCase(String str) {
			return str.toLowerCase().replaceAll(" ", "") ;
		}
		
		public String toCamelCase(String str) {
			NamingServices namingservice = new NamingServices() ;
			String[] words = str.split(" ");
			String camelCasedStr = "" ;
			for(int i=0;i<words.length;i++) {
				camelCasedStr += words[i].substring(0, 1).toUpperCase()+namingservice.toSqlCase(words[i].substring(1)) ;
			}
			String[] words2 = str.split("_");
			String camelCasedStr2 = "" ;
			for(int i=0;i<words2.length;i++) {
				camelCasedStr2 += words2[i].substring(0, 1).toUpperCase()+namingservice.toSqlCase(words2[i].substring(1)) ;
			}
			return camelCasedStr2 ;
		}
		
		public String toFileName(String str) {
			return str.toLowerCase().replaceAll(" ", "_") ;
		}
		
        public String SqlTableName(ModuleElement cl) {
        	return this.toSqlCase(cl.getModule().getName())+"_"+this.toSqlCase(cl.getName()) ;
        }
        
        public String SqlTableName(Attribute attr) {
        	return this.SqlTableName(attr.getClass_())+"_"+this.toSqlCase(attr.getName()) ;
        }
        
        public String SqlTableName(BinaryAssociation asso) {
        	if(asso.getName().equals("")) {
	        	Iterator it = asso.getAssociation_ends().iterator() ;
	        	ArrayList association_ends = new ArrayList() ;
	        	while(it.hasNext()) {
	        		association_ends.add(((AssociationEnd)it.next()).getName()) ;
	        	}
	        	Collections.sort(association_ends) ;
	        	Iterator end = association_ends.iterator() ;
	        	String table_name = this.toSqlCase(asso.getModule().getName()) ;
	        	while(end.hasNext()) {
	        		table_name += "_"+this.toSqlCase((String)end.next()) ;
	        	}
	        	return table_name ;
        	} else {
        		return this.toSqlCase(asso.getModule().getName())+"_"+this.toSqlCase(asso.getName()) ;
        	}
        }
        
        public String SqlColumnName(Attribute attr) {
        	return this.toSqlCase(attr.getName()) ;
        }
        
        public String SqlColumnQualifiedName(Attribute attr) {
    		return this.SqlTableName(attr.getClass_())+"."+this.toSqlCase(attr.getName()) ;
        }
        
        public String SqlColumnName(AssociationEnd end) {
        	return this.toSqlCase(end.getName()) ;
        }
        
        public String SqlColumnQualifiedName(AssociationEnd end) {
        	SqlServices sql = new SqlServices() ;
        	AssociationServices asso = new AssociationServices() ;
        	if(sql.implyForeignKeyForAssoc(asso.getOppositeEnd(end))) {
        		return this.SqlTableName((ModuleElement)(asso.getOppositeEnd(end).getType()))+"."+this.toSqlCase(end.getName()) ;
        	} else {
        		return this.SqlTableName((ModuleElement)(end.getType()))+"."+this.toSqlCase(end.getName()) ;
        	}
        }
        
        public String ClassName(Class cl) {
        	return this.toCamelCase(cl.getName()) ;
        }
        
        public String ClassName(Association asso) {
        	return this.toCamelCase(asso.getName()) ;
        }
        
        public String PhpDtoClassName(Class cl) {
        	return this.ModuleName(cl.getModule())+"_Dto_"+this.ClassName(cl) ;
        }
        
        public String PhpDtoClassName(Association asso) {
        	return this.ModuleName(asso.getModule())+"_Dto_"+this.ClassName(asso) ;
        }
        
        public String PhpDaoClassName(Class cl,String persistence) {
        	return this.ModuleName(cl.getModule())+"_Dao_"+persistence+"_"+this.ClassName(cl) ;
        }
        
        public String PhpDaoClassName(Association asso,String persistence) {
        	return this.ModuleName(asso.getModule())+"_Dao_"+persistence+"_"+this.ClassName(asso) ;
        }
        
        public String ModuleName(Module module) {
        	return this.toSqlCase(module.getName()).toLowerCase();
        }
        
        public String ModuleName(ModuleElement el) {
        	return this.toSqlCase(el.getModule().getName()).toLowerCase();
        }
        
        public String DirName(Module module) {
        	return this.toSqlCase(module.getName()).toLowerCase();
        }
        
        public String FileName(Class cl) {
        	return this.toCamelCase(cl.getName());
        }
        
        public String FileName(Association asso) {
        	return this.toCamelCase(asso.getName());
        }
        
        public String AttributeName(Attribute attr) {
        	return this.toSqlCase(attr.getName()) ;
        }
        
        public String AttributeName(AssociationEnd end) {
        	return this.toSqlCase(end.getName()) ;
        }
        
        public String toPhpVariable(String str) {
			return str.toLowerCase().replaceAll(" ", "_") ;
        }
        
        public String toPhpVariable(Attribute attr) {
        	return "$"+this.toPhpVariable(attr.getName()) ;
        }
        
        public String toPhpVariable(AssociationEnd end) {
        	return "$"+this.toPhpVariable(end.getName()) ;
        }
}