package fr.obeo.weblog.dao.blog.api;

// Start of user code for import
import java.util.Collection;
import java.util.List;

import fr.obeo.fwk.dao.exception.DaoException;
import fr.obeo.weblog.business.blog.User;
// End of user code for import

/**
 * This class provides the data access layer to the User entity class.<br/>
 * This is the interface which represent the contrat of the DAO access.
 */
public interface IUserDao {

	/**
	 * Create a new element
	 * @param user Element to create.
	 */
	public void createUser(User user) throws DaoException;

	/**
	 * Update an existing element
	 * @param user Element to update. If the element has an id, it may use it.
	 */
	public void updateUser(User user) throws DaoException;

	/**
	 * Delete an element
	 * @param user Element which will be delete. 
	 * Only id can be used to find which element must be deleted.
	 */
	public void deleteUser(User user) throws DaoException;

	/**
	 * Find all elements.
	 * @return A list with all elements, without any filter.
	 */
	public Collection<User> findAllUsers() throws DaoException;

	/**
	 * Find one entity by its primary key.
	 * @return The entity found.
	 */
	public User findUserById(String id) throws DaoException;

	/**
	 * Get a User according to its login and password
	 * @param login 
	 * @param password 
	 * @return 
	 */
	public User findByLoginAndPassword(String login, String password) throws DaoException;

	//Start of user code for technicals dao access api
	//End of user code
}