package fr.obeo.weblog.dao.blog.test;

// Start of user code for import 
import java.util.Date;
import org.hibernate.Transaction;
import junit.framework.Assert;
import junit.framework.TestCase;

import fr.obeo.fwk.dao.exception.DaoException;
import fr.obeo.fwk.dao.hibernate.HibernateUtil;
import fr.obeo.weblog.dao.WeblogDaoFactory;
import fr.obeo.weblog.dao.blog.api.IBlogEntryDao;
import fr.obeo.weblog.business.blog.BlogEntry;
import fr.obeo.weblog.business.blog.User;

// End of user code for import

public class IBlogEntryDaoTest extends TestCase {

	private IBlogEntryDao dao;
	private Transaction tx;

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		dao = WeblogDaoFactory.getBlogEntryDao();
		tx = HibernateUtil.currentSession().beginTransaction();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
		tx.commit();
		HibernateUtil.closeSession();
	}

	/**
	 * Test the creation of the entity BlogEntry.<br/>
	 * <ul><li>Step 1 : Create an entity</li>
	 * <li>Step 2 : Search the entity and verify it exist</li></ul>
	 */
	public final void testCreateBlogEntry() throws DaoException {
        // fill attributes with example values
		BlogEntry blogEntry = getBlogEntryExample();
		
        // Execute the tested code
		dao.createBlogEntry(blogEntry);

		// Verify result
		boolean found = false;
		for(BlogEntry currentBlogEntry : dao.findAllBlogEntrys()) {
			if (currentBlogEntry.equals(blogEntry)) {
				found = true;
			}
		}
		Assert.assertTrue("BlogEntry not created", found);
	}

	/**
	 * Test the modification of an entity BlogEntry.<br/>
	 * <ul><li>Step 1 : Create an entity</li>
	 * <li>Step 2 : Modify the entity</li>
	 * <li>Step 3 : Search the entity and verify the modified values</li></ul>
	 */
	public final void testUpdateBlogEntry() throws DaoException {
        // Initialized the test
		BlogEntry blogEntry = getBlogEntryExample();
		dao.createBlogEntry(blogEntry);

        // Execute the tested code
		blogEntry.setTitle("secondTitle");
		blogEntry.setCreationDate(new Date(1));
		blogEntry.setContent("secondContent");
		dao.updateBlogEntry(blogEntry);

		// Verify result
		boolean found = false;
		for(BlogEntry currentBlogEntry : dao.findAllBlogEntrys()) {
			if (currentBlogEntry.equals(blogEntry)) {
				found = true;
				Assert.assertEquals("Value title not modified", "secondTitle", blogEntry.getTitle());
				Assert.assertEquals("Value creationDate not modified", new Date(1), blogEntry.getCreationDate());
				Assert.assertEquals("Value content not modified", "secondContent", blogEntry.getContent());
			}
		}
		Assert.assertTrue("BlogEntry not found", found);
	}

	/**
	 * Test the suppression of an entity BlogEntry.<br/>
	 * <ul><li>Step 1 : Create an entity</li>
	 * <li>Step 2 : Delete the entity</li>
	 * <li>Step 3 : Search the entity and verify it doesn't exist anymore</li></ul>
	 */
	public final void testDeleteBlogEntry() throws DaoException {
        // Initialized the test
		BlogEntry blogEntry = getBlogEntryExample();
		dao.createBlogEntry(blogEntry);
		
        // Execute the tested code
		dao.deleteBlogEntry(blogEntry);
		
		// Verify result
		boolean found = false;
		for(BlogEntry currentBlogEntry : dao.findAllBlogEntrys()) {
			if (currentBlogEntry.equals(blogEntry)) {
				found = true;				
			}
		}
		Assert.assertFalse("BlogEntry not deleted", found);
	}

	/**
	 * Test the search of all entities BlogEntry.<br/>
	 * <ul><li>Step 1 : Create many entities</li>
	 * <li>Step 2 : Search all entities and verify the entities created are found</li></ul>
	 */
	public final void testFindAllBlogEntrys() throws DaoException {
		// Verify number of element before testing
		int before = dao.findAllBlogEntrys().size();
		Assert.assertTrue("FindAll must return at least 0", before >= 0);
		
        // Add two elements
		BlogEntry blogEntry1 = getBlogEntryExample();
		dao.createBlogEntry(blogEntry1);
				
		BlogEntry blogEntry2 = getBlogEntryExample();
		dao.createBlogEntry(blogEntry2);

		// Verify result
		int after = dao.findAllBlogEntrys().size();
		Assert.assertEquals(
				"FindAll don't find the good number of elements", 
				2,
				after - before);
	}



	/**
	 * Create an example of BlogEntry
	 */
	protected BlogEntry getBlogEntryExample() throws DaoException {
		// fill attributes with example values
		BlogEntry blogEntry = new BlogEntry();
		blogEntry.setTitle("myTitle");
		blogEntry.setCreationDate(new Date(0));
		blogEntry.setContent("myContent");
			
		// Start of user code for example of User.authorUser values
		User authorUserExample = new User();
		blogEntry.setAuthorUser(authorUserExample);
		WeblogDaoFactory.getUserDao().createUser(authorUserExample);
		// End of user code for example of User.authorUser values
		
		return blogEntry;
	}
	
	/**
	 * Create a second example of BlogEntry with other values
	 */
	protected BlogEntry getBlogEntryExample2() throws DaoException {
		// fill attributes with example values
		BlogEntry blogEntry = new BlogEntry();
		blogEntry.setTitle("secondTitle");
		blogEntry.setCreationDate(new Date(1));
		blogEntry.setContent("secondContent");
			
		// Start of user code for example 2 of User.authorUser values
		User authorUserExample = new User();
		blogEntry.setAuthorUser(authorUserExample);
		WeblogDaoFactory.getUserDao().createUser(authorUserExample);
		// End of user code for example 2 of User.authorUser values
		
		return blogEntry;
	}
	
	// Start of user code of specific tests
	// End of user code of specific tests

}
