package org.acceleo.sample.demo.spring.entities;

// Start of user code for import
import java.io.Serializable;

// End of user code

/**
 * 
 */
public class Product implements Serializable {

	// serialVersionUID is used for serialization.
	private static final long serialVersionUID = 1L;

	public final static String PROP_ID = "id";
	public final static String PROP_NAME = "name";
	public final static String PROP_COLOR = "color";

	private String id;
	protected String name;
	protected String color;

	/**
	 * Default constructor 
	 */
	public Product() {
		super();
	}
	
	/**
	 * Return the identifier
	 * @return Id
	 */
	public String getId() {
		return this.id;
	}

	/**
	 * Set a value to parameter Id
	 * @param id Value of the identifier
	 */
	public void setId(String id) {
		this.id = id;
	}
	

	/**
	 * Constructor with all parameters initialized
	 * @param name 
	 * @param color 
	 */
	public Product(String name, String color) {
		this.name = name;
		this.color = color;
	}	

	/**
	 * Return name
	 * @return 
	 */
	public String getName() {
		return name;
	}

	/**
	 * Set a value to parameter name.
	 * @param name 
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Return color
	 * @return 
	 */
	public String getColor() {
		return color;
	}

	/**
	 * Set a value to parameter color.
	 * @param color 
	 */
	public void setColor(String color) {
		this.color = color;
	}


	
	/** 
	 * Equality test based attributes values
	 * @param value Value to compare
	 */
	public boolean equals(Object other) {
		if (this == other) {
			return true;
		}
		if ( ! (other instanceof Product)) {
			return false;
		}
		
		final Product castedOther = (Product) other;
		
		if ( id != null && castedOther.getId() != null ) {
			return id.equals( castedOther.getId() );
		}
		
		if ( (	name == null && castedOther.getName() != null) || 
			 (	name != null && !name.equals(castedOther.getName()) ) 
		   ) return false;
		if ( (	color == null && castedOther.getColor() != null) || 
			 (	color != null && !color.equals(castedOther.getColor()) ) 
		   ) return false;
		
		return true;
	}

	/** 
	 * HashTable code based on attributes hashcodes
	 * @return hash value
	 */
	public int hashCode() {
		int result = 0;
		if (id != null) result = id.hashCode();
		if (name != null) result = 29 * result + name.hashCode();
		if (color != null) result = 29 * result + color.hashCode();
		return result;
	}
   
	// Start of user code for private methods
	// End of user code
   
}