package org.acceleo.sample.demo.spring.daos.impl;



import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.junit.Test;

import org.acceleo.sample.demo.spring.AbstractBusinessLayerTests;

import org.acceleo.sample.demo.spring.daos.IProductsDao;

import org.acceleo.sample.demo.spring.entities.Product;

import org.acceleo.sample.demo.spring.entities.impl.ProductHelper;



// Start of user code for import 
import java.util.Set;

// End of user code for import

public class ProductsDaoTest extends AbstractBusinessLayerTests {

	/**
	 * class dedicated logger
 	 */
	private static Log log = LogFactory.getLog(ProductsDaoTest.class.getName()); 

	private IProductsDao productsDao;
	
	@Override
	protected void onSetUp() throws Exception {
		super.onSetUp();		
				
		this.productsDao = getProductsDao();
		assertNotNull(productsDao);
		// Start of user code of onSetUp

		// End of user code of onSetUp	 
	}

	@Override
	protected void onTearDown() throws Exception {
		super.onTearDown();
		// Start of user code of onTearDown

		// End of user code of onTearDown
	}
		
	protected IProductsDao getProductsDao() throws Exception {
		IProductsDao productsDao = (IProductsDao)getApplicationContext().getBean("productsDao");
		assertNotNull(productsDao);		
		return productsDao;
	}

	/**
	 * Test the creation of the entity ProductsDao.<br/>
	 * <ul><li>Step 1 : Create an entity</li>
	 * <li>Step 2 : Search the entity and verify it exist</li></ul>
	 */
	@Test
	public final void testCreateProductsDao() throws Exception {        
        // fill attributes with exemple values
		Product product = ProductHelper.createProduct();
		
        // Execute the tested code
		productsDao.create(product);
		getSession().flush();

		// Verify result
		boolean found = false;
		for(Product currentProduct : productsDao.findAll()) {
			if (currentProduct.equals(product)) {
				found = true;
			}
		}
		assertTrue("Product not created", found);
		
	}
	
	/**
	 * Test the modification of an entity ProductsDao.<br/>
	 * <ul><li>Step 1 : Create an entity</li>
	 * <li>Step 2 : Modify the entity</li>
	 * <li>Step 3 : Search the entity and verify the modified values</li></ul>
	 */
	@Test
	public final void testSaveProductsDao() throws Exception {
        // Initialized the test
		Product product = ProductHelper.createProduct();
		productsDao.create(product);		
		
        // Execute the tested code
					product.setName("secondName");
					product.setColor("secondColor");
        
		productsDao.save(product);
		getSession().flush();

		// Verify result
		boolean found = false;
		for(Product currentProduct : productsDao.findAll()) {
			if (currentProduct.equals(product)) {
				found = true;
			}
		}
		
		assertTrue("Product not found", found);
	}
	
	/**
	 * Test the suppression of an entity ProductsDao.<br/>
	 * <ul><li>Step 1 : Create an entity</li>
	 * <li>Step 2 : Delete the entity</li>
	 * <li>Step 3 : Search the entity and verify it doesn't exist anymore</li></ul>
	 */
	@Test
	public final void testDeleteProductsDao() throws Exception {
        // Initialized the test
		Product product = ProductHelper.createProduct();
		productsDao.create(product);
    	
        // Execute the tested code
		productsDao.delete(product);		
		getSession().flush();
		
		// Verify result
		boolean found = false;		
		for(Product currentProduct : productsDao.findAll()) {
			if (currentProduct.equals(product)) {
				found = true;		
			}
		}
		
		assertFalse("ProductsDao not deleted", found);
	}
	
	

	// Start of user code of specific tests
	/**
	 * Another test
	 */
	public final void testfindProductsByColor() throws Exception {
		// Initialized the test
		Product product = ProductHelper.createProduct();
		productsDao.create(product);
		Set<Product> products = productsDao.findProductsByColor("blue");
		assertNotNull(products);
		int size = products.size();

		// Execute the tested code
		product.setColor("blue");
		productsDao.update(product);
		getSession().flush();
		products = productsDao.findProductsByColor("blue");

		// Verify result
		assertNotNull(products);
		assertEquals(products.size(), size + 1);

	}

	// End of user code of specific tests

}